'use strict';

var vue = require('vue');
var index$1 = require('../_util/dom-align/index.js');
var index = require('../_util/props-util/index.js');
var util = require('./util.js');
var core = require('@vueuse/core');

const alignProps = {
  align: Object,
  target: [Object, Function],
  onAlign: Function,
  disabled: [Boolean, Object],
  resizeObservable: {
    type: [Boolean, Object],
    default: true
  }
};
function getElement(ins) {
  const el = core.unrefElement(ins);
  return el instanceof HTMLElement ? el : void 0;
}
function getPoint(point) {
  const el = core.unrefElement(point);
  if (el instanceof HTMLElement) {
    const {
      offsetLeft: pageX,
      offsetTop: pageY
    } = getElement(point);
    return {
      pageX,
      pageY
    };
  } else {
    return point;
  }
}
var Align = vue.defineComponent({
  name: "Align",
  props: alignProps,
  setup(props, {
    slots,
    expose
  }) {
    const sourceRef = vue.ref();
    function forceAlign() {
      if (vue.unref(props.disabled))
        return;
      const {
        onAlign,
        align
      } = props;
      const source = index.findDOMNode(sourceRef.value);
      if (props.target && source) {
        let result;
        const target = getElement(props.target), point = getPoint(props.target);
        targetResizeMonitor.el = target;
        targetResizeMonitor.point = point;
        if (target) {
          result = index$1["default"](source, target, vue.unref(align));
        } else if (point) {
          result = index$1.alignPoint(source, point, vue.unref(align));
        }
        if (onAlign && result) {
          onAlign(source, result);
        }
      }
    }
    const targetResizeMonitor = {
      el: null,
      point: null,
      cancel: () => {
      }
    };
    const sourceResizeMonitor = {
      el: null,
      cancel: () => {
      }
    };
    function goAlign() {
      if (vue.unref(props.disabled))
        return;
      const source = index.findDOMNode(sourceRef.value);
      if (source !== sourceResizeMonitor.el) {
        sourceResizeMonitor.cancel();
        sourceResizeMonitor.el = source;
      }
      const target = getElement(props.target), point = getPoint(props.target);
      if (targetResizeMonitor.el != target || !util.isSamePoint(targetResizeMonitor.point, point)) {
        forceAlign();
        if (targetResizeMonitor.el != target) {
          targetResizeMonitor.cancel();
          targetResizeMonitor.el = target;
          targetResizeMonitor.point = point;
          targetResizeMonitor.cancel = util.monitorResize(target, forceAlign);
        }
      }
    }
    vue.onUpdated(() => {
      goAlign();
    });
    vue.onMounted(() => {
      goAlign();
    });
    vue.onUnmounted(() => {
      targetResizeMonitor.cancel();
      sourceResizeMonitor.cancel();
    });
    expose({
      forceAlign
    });
    return () => {
      var _a;
      const child = (_a = slots.default) == null ? void 0 : _a.call(slots);
      if (child) {
        return vue.cloneVNode(child[0], {
          ref: sourceRef
        }, true);
      }
    };
  }
});

module.exports = Align;
