'use strict';

var vue = require('vue');
var core = require('@vueuse/core');
var getScrollParent = require('../_util/Dom/getScrollParent.js');
require('../_util/hooks/index.js');
var _ = require('../_util/_.js');
var context = require('./context.js');
var utils = require('./utils.js');
var useNameSpace = require('../_util/hooks/useNameSpace2.js');

const preCls = "x-anchor";
const anchorProps = {
  value: String,
  type: { type: String, default: "rail" },
  showBackground: { type: Boolean, default: true },
  offset: { type: Number, default: 0 },
  container: [String, Object, Function],
  scrollDuration: { type: Number, default: 350 }
};
var _sfc_main = vue.defineComponent({
  name: preCls,
  props: anchorProps,
  emits: ["change"],
  setup(props, { emit }) {
    const ns = useNameSpace(preCls);
    const anchorRef = vue.ref();
    const barRef = vue.ref();
    const activeRef = vue.ref("");
    const links = [];
    vue.watch(activeRef, (v) => emit("change", v));
    vue.watchPostEffect(() => {
      var _a;
      if (!barRef.value)
        return;
      const el = (_a = links.find((e) => e.href == activeRef.value)) == null ? void 0 : _a.el;
      if (el) {
        const boundTop = anchorRef.value.getBoundingClientRect().top;
        const rect = el.getBoundingClientRect();
        const h = rect.height;
        barRef.value.style.height = h + "px";
        barRef.value.style.width = "100%";
        barRef.value.style.top = rect.top - boundTop + (h >> 1) + "px";
      } else {
        barRef.value.style.height = "0px";
        barRef.value.style.width = "0px";
      }
    });
    vue.provide(context.AnchorContextKey, {
      value: activeRef,
      links,
      setActive,
      props
    });
    const container = vue.computed(() => {
      return props.container ? typeof props.container == "string" ? document.querySelector(props.container) : core.unrefElement(props.container) : getScrollParent.getScrollParent(anchorRef.value);
    });
    const onScroll = _.throttle(computeCurrentLink, 128);
    const addListener = () => {
      var _a;
      (_a = container.value) == null ? void 0 : _a.addEventListener("scroll", onScroll, { passive: true });
    };
    const delListener = () => {
      container.value.removeEventListener("scroll", onScroll);
      onScroll.cancel();
    };
    vue.onMounted(() => {
      vue.watch(
        container,
        (val, old) => {
          old == null ? void 0 : old.removeEventListener("scroll", onScroll);
          addListener();
        },
        { immediate: true, flush: "post" }
      );
    });
    vue.onBeforeUnmount(() => {
      delListener();
    });
    function getBoundTop() {
      return container.value.nodeType != 9 ? container.value.getBoundingClientRect().top : 0;
    }
    const scrollEl = vue.computed(() => container.value.nodeType == 9 ? document.documentElement : container.value);
    const scroller = utils.scroller(scrollEl, 350, delListener, addListener);
    function setActive(href) {
      activeRef.value = href;
      const el = container.value.querySelector(href);
      if (!el)
        return;
      const rect = el.getBoundingClientRect();
      const y = scrollEl.value.scrollTop + (rect.top - getBoundTop()) - props.offset + 1;
      scroller.scroll({ y });
      location.hash = encodeURIComponent(href.replace(/^#/, ""));
    }
    function computeCurrentLink() {
      if (scroller.transiting)
        return;
      const { offset } = props;
      const boundTop = getBoundTop();
      const infos = links.map((e) => {
        const el = container.value.querySelector(e.href);
        return {
          href: e.href,
          rect: el == null ? void 0 : el.getBoundingClientRect()
        };
      }).filter((e) => e.rect).sort((a, b) => a.rect.top - b.rect.top);
      let info;
      for (let i = 0; i < infos.length; i++) {
        if (infos[i].rect.top <= boundTop + offset)
          info = infos[i];
        else
          break;
      }
      activeRef.value = info == null ? void 0 : info.href;
    }
    return {
      ns,
      anchorRef,
      barRef
    };
  }
});

module.exports = _sfc_main;
