'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
var moment = require('moment');
var _ = require('../_util/_.js');
var CalendarHeader = require('./CalendarHeader.js');
var CalendarGrid = require('./CalendarGrid.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var moment__default = /*#__PURE__*/_interopDefaultLegacy(moment);

const dateCls = "x-calendar";
const weekdays = ["\u65E5", "\u4E00", "\u4E8C", "\u4E09", "\u56DB", "\u4E94", "\u516D"];
const DateCalendar = vue.defineComponent({
  name: dateCls,
  props: {
    value: [Object, Array],
    range: Array,
    disabledDate: Function,
    onSelect: Function,
    onHover: Function
  },
  emits: ["update:value"],
  setup(props, {
    slots,
    emit
  }) {
    var _a, _b, _c;
    const val1 = (_b = _.castArray(props.value)[0]) != null ? _b : (_a = props.range) == null ? void 0 : _a[0];
    const date1 = ((_c = val1 == null ? void 0 : val1.clone()) != null ? _c : moment__default["default"]()).date(1);
    const start = date1.add(moment__default["default"].localeData().firstDayOfWeek() - date1.day(), "days");
    const display = vue.ref(date1.clone());
    const gridRef = vue.ref();
    const visible = vue.ref(false);
    function showPanel() {
      visible.value = true;
    }
    function onSelectGroup(m) {
      visible.value = false;
      gridRef.value.scroll(m, 0);
    }
    function onSelect(date) {
      var _a2;
      emit("update:value", date);
      (_a2 = props.onSelect) == null ? void 0 : _a2.call(props, date);
    }
    return () => {
      var _a2, _b2;
      return vue.createVNode("div", {
        "class": dateCls
      }, [vue.createVNode(CalendarHeader, {
        "prefixCls": dateCls,
        "title": display.value.format("YYYY\u5E74M\u6708"),
        "onPrev": (_a2 = gridRef.value) == null ? void 0 : _a2.prev,
        "onNext": (_b2 = gridRef.value) == null ? void 0 : _b2.next,
        "onPanelChange": showPanel
      }, null), vue.createVNode("div", {
        "class": `${dateCls}_grid ${dateCls}_week`
      }, [weekdays.map((day) => vue.createVNode("div", {
        "title": day,
        "class": `${dateCls}_weekday`
      }, [day]))]), vue.createVNode(CalendarGrid.DateGrid, {
        "_ref": gridRef,
        "prefixCls": dateCls,
        "start": start,
        "modelValue": props.value,
        "rangeValue": props.range,
        "disabled": props.disabledDate,
        "cell": (date) => {
          var _a3, _b3;
          return (_b3 = (_a3 = slots["date-cell"] || slots.dateCell) == null ? void 0 : _a3({
            date
          })) != null ? _b3 : CalendarGrid.dateProps.cell(date);
        },
        "onSelect": onSelect,
        "onHover": props.onHover,
        "onGroupChange": (date) => display.value = date
      }, null), vue.createVNode(vue.Transition, {
        "name": "calendar-ain"
      }, {
        default: () => [visible.value ? vue.createVNode(MonthCalendar, {
          "value": props.value,
          "display": display.value,
          "style": "position: absolute; top: 0; left: 0; width: 100%; height: 100%",
          "cell": (date) => {
            var _a3, _b3;
            return (_b3 = (_a3 = slots["month-cell"] || slots.monthCell) == null ? void 0 : _a3({
              date
            })) != null ? _b3 : CalendarGrid.monthProps.cell(date);
          },
          "yearCell": (date) => {
            var _a3, _b3;
            return (_b3 = (_a3 = slots["year-cell"] || slots.yearCell) == null ? void 0 : _a3({
              date
            })) != null ? _b3 : CalendarGrid.yearProps.cell(date);
          },
          "onSelect": onSelectGroup
        }, null) : null]
      })]);
    };
  }
});
const monthCls = "x-month-calendar";
const MonthCalendar = vue.defineComponent({
  name: monthCls,
  props: {
    value: [Object, Array],
    range: Array,
    display: Object,
    disabledDate: Function,
    cell: Function,
    yearCell: Function,
    onSelect: Function,
    onHover: Function
  },
  setup(props) {
    var _a, _b, _c, _d, _e;
    const val1 = (_b = _.castArray(props.value)[0]) != null ? _b : (_a = props.range) == null ? void 0 : _a[0];
    const start = ((_e = (_d = (_c = props.display) == null ? void 0 : _c.clone()) != null ? _d : val1 == null ? void 0 : val1.clone()) != null ? _e : moment__default["default"]()).month(0);
    const display = vue.ref(start.clone());
    const gridRef = vue.ref();
    const visible = vue.ref(false);
    function showPanel() {
      visible.value = true;
    }
    function onSelectGroup(m) {
      visible.value = false;
      gridRef.value.scroll(m, 0);
    }
    return () => {
      var _a2, _b2, _c2;
      return vue.createVNode("div", {
        "class": monthCls
      }, [vue.createVNode(CalendarHeader, {
        "prefixCls": monthCls,
        "title": display.value.format("YYYY\u5E74"),
        "onPrev": (_a2 = gridRef.value) == null ? void 0 : _a2.prev,
        "onNext": (_b2 = gridRef.value) == null ? void 0 : _b2.next,
        "onPanelChange": showPanel
      }, null), vue.createVNode(CalendarGrid.MonthGrid, {
        "_ref": gridRef,
        "prefixCls": monthCls,
        "start": start,
        "modelValue": props.value,
        "rangeValue": props.range,
        "disabled": props.disabledDate,
        "cell": (_c2 = props.cell) != null ? _c2 : CalendarGrid.monthProps.cell,
        "onSelect": props.onSelect,
        "onHover": props.onHover,
        "onGroupChange": (date) => display.value = date
      }, null), vue.createVNode(vue.Transition, {
        "name": "calendar-ain"
      }, {
        default: () => {
          var _a3;
          return [visible.value ? vue.createVNode(YearCalendar, {
            "value": props.value,
            "display": display.value,
            "style": "position: absolute; top: 0; left: 0; width: 100%; height: 100%",
            "cell": (_a3 = props.yearCell) != null ? _a3 : CalendarGrid.yearProps.cell,
            "onSelect": onSelectGroup
          }, null) : null];
        }
      })]);
    };
  }
});
const YearCalendar = vue.defineComponent({
  name: "x-year-calendar",
  props: {
    value: [Object, Array],
    range: Array,
    display: Object,
    disabledDate: Function,
    cell: Function,
    onSelect: Function,
    onHover: Function
  },
  setup(props) {
    var _a, _b, _c, _d, _e;
    const val1 = (_b = _.castArray(props.value)[0]) != null ? _b : (_a = props.range) == null ? void 0 : _a[0];
    const start = group((_e = (_d = (_c = props.display) == null ? void 0 : _c.clone()) != null ? _d : val1 == null ? void 0 : val1.clone()) != null ? _e : moment__default["default"]());
    const display = vue.ref(start.clone());
    const title = vue.computed(() => group(display.value).year() + " - " + group(display.value, 9).year());
    const gridRef = vue.ref();
    function group(date, amount = 0) {
      return date.clone().add(-date.year() % 10 + amount, "years");
    }
    return () => {
      var _a2, _b2, _c2;
      return vue.createVNode("div", {
        "class": monthCls
      }, [vue.createVNode(CalendarHeader, {
        "prefixCls": monthCls,
        "title": title.value,
        "onPrev": (_a2 = gridRef.value) == null ? void 0 : _a2.prev,
        "onNext": (_b2 = gridRef.value) == null ? void 0 : _b2.next
      }, null), vue.createVNode(CalendarGrid.YearGrid, {
        "_ref": gridRef,
        "prefixCls": monthCls,
        "start": start,
        "modelValue": props.value,
        "rangeValue": props.range,
        "disabled": props.disabledDate,
        "cell": (_c2 = props.cell) != null ? _c2 : CalendarGrid.yearProps.cell,
        "onSelect": props.onSelect,
        "onHover": props.onHover,
        "onGroupChange": (date) => display.value = date
      }, null)]);
    };
  }
});

exports.DateCalendar = DateCalendar;
exports.MonthCalendar = MonthCalendar;
exports.YearCalendar = YearCalendar;
