'use strict';

var vue = require('vue');
var core = require('@vueuse/core');
var _ = require('../_util/_.js');
var ensureRange = require('../_util/ensureRange.js');

var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
var Scroll = vue.defineComponent({
  inheritAttrs: false,
  props: {
    initial: Number,
    max: {
      type: Number,
      default: Infinity
    },
    onScroll: Function
  },
  setup(props, {
    slots,
    attrs,
    expose
  }) {
    const el = vue.ref();
    const state = vue.reactive({
      startI: 0,
      endI: 1
    });
    const onScroll = _.throttle(props.onScroll, 200);
    vue.onBeforeUnmount(() => onScroll.cancel());
    let itemH = 0;
    let warpH = 0;
    vue.onMounted(() => {
      const firstChild = el.value.firstElementChild;
      const style = window.getComputedStyle(firstChild);
      itemH = firstChild.offsetHeight + parseInt(style.marginTop) + parseInt(style.marginBottom);
      warpH = el.value.offsetHeight;
      state.endI = Math.ceil(warpH / itemH);
    });
    let y = 0;
    function onWheel(e) {
      if (props.initial != null && y == -props.initial)
        return;
      const deltaY = ensureRange(e.deltaY, -50, 50);
      y = props.initial == null ? y + deltaY : ensureRange(y + deltaY, -props.initial, props.max - props.initial);
      e.stopPropagation();
      e.preventDefault();
      doScroll(y);
    }
    function doScroll(top) {
      y = top;
      Object.assign(state, calc(top));
      el.value.style.transform = `translateY(${-top + state.startI * itemH}px)`;
      onScroll({
        y: top
      });
    }
    function calc(y2) {
      return {
        startI: Math.floor(y2 / itemH),
        endI: Math.ceil((y2 + warpH) / itemH)
      };
    }
    let st = 0, sy = 0, ey = 0, d = 200;
    const {
      resume,
      pause
    } = core.useRafFn(() => {
      const rate = Math.min((Date.now() - st) / d, 1);
      doScroll(sy + (ey - sy) * rate);
      if (rate == 1)
        return pause();
    });
    function scroll(top, t = 200) {
      return __async(this, null, function* () {
        pause();
        sy = y;
        ey = top;
        st = Date.now();
        d = t;
        resume();
      });
    }
    function scrollIndex(i, t) {
      scroll(i * itemH, t);
    }
    expose({
      calc,
      scroll,
      scrollIndex
    });
    const genClass = () => vue.normalizeClass(attrs.class).split(" ").map((e) => `${e}-scroll`);
    const genItem = (_, i) => slots.default({
      index: state.startI + i
    });
    return () => vue.createVNode("div", {
      "class": genClass(),
      "style": "overflow: hidden;",
      "onWheel": onWheel
    }, [vue.createVNode("div", vue.mergeProps(attrs, {
      "ref": el
    }), [Array.from(Array(state.endI - state.startI), genItem).flat()])]);
  }
});

module.exports = Scroll;
