'use strict';

var vue = require('vue');
var index = require('../border-light/index.js');
require('../icon/index.js');
var index$1 = require('../_util/props-util/index.js');
var Icon = require('../icon/Icon.js');

const Card = vue.defineComponent({
  name: "x-card",
  props: {
    prefixCls: {
      default: "x-card"
    },
    title: String,
    icon: Object,
    bordered: {
      default: true
    },
    hoverable: Boolean,
    closable: Boolean,
    contentClass: [Object, String, Array],
    contentStyle: [Object, String],
    footerClass: [Object, String, Array],
    footerStyle: [Object, String],
    headerClass: [Object, String, Array],
    headerStyle: [Object, String],
    size: String
  },
  slots: ["title", "cover", "header", "header-extra", "default", "footer"],
  emits: ["close"],
  directives: {
    [index.name]: index
  },
  setup(props, {
    emit,
    slots
  }) {
    function close(e) {
      e.stopPropagation();
      e.preventDefault();
      emit("close");
    }
    return (vm) => {
      var _a, _b, _c, _d, _e;
      const {
        prefixCls,
        size,
        closable,
        contentClass,
        footerClass,
        headerClass,
        icon
      } = props;
      const cls = [prefixCls, {
        [`${prefixCls}-bordered`]: props.bordered,
        [`${prefixCls}-hoverable`]: props.hoverable,
        [`${prefixCls}-${size}`]: size
      }];
      const coverNode = (_a = slots.cover) == null ? void 0 : _a.call(slots);
      const titleNode = index$1.getComponent(vm, "title");
      const closeNode = closable ? vue.createVNode(Icon["default"], {
        "class": `${prefixCls}_close`,
        "type": "cross",
        "onClick": close
      }, null) : void 0;
      const contentNode = (_b = slots.default) == null ? void 0 : _b.call(slots);
      const footerNode = (_c = slots.footer) == null ? void 0 : _c.call(slots);
      const iconNode = index$1.flattenChildren(index$1.getComponent(vm, "icon"))[0];
      const headerExtraNode = (_d = slots["header-extra"]) == null ? void 0 : _d.call(slots);
      const headerNode = slots.header ? vue.createVNode("div", {
        "class": [`${prefixCls}_header`, headerClass]
      }, [(_e = slots.header) == null ? void 0 : _e.call(slots)]) : vue.createVNode("div", {
        "class": `${prefixCls}_header`,
        "style": props.headerStyle
      }, [index$1.hasElemnt(iconNode) && vue.cloneVNode(iconNode, {
        class: `${prefixCls}_icon`
      }), vue.createVNode("div", {
        "class": `${prefixCls}_title`
      }, [titleNode]), headerExtraNode && vue.createVNode("div", {
        "class": `${prefixCls}_header_extra`
      }, [headerExtraNode]), closeNode]);
      return vue.withDirectives(vue.createVNode("div", {
        "class": cls
      }, [index$1.hasElemnt(coverNode) && vue.createVNode("div", {
        "class": `${prefixCls}_cover`
      }, [coverNode]), headerNode, index$1.hasElemnt(contentNode) && vue.createVNode("div", {
        "class": [`${prefixCls}_content`, contentClass],
        "style": props.contentStyle
      }, [contentNode]), index$1.hasElemnt(footerNode) && vue.createVNode("div", {
        "class": [`${prefixCls}_footer`, footerClass],
        "style": props.footerStyle
      }, [footerNode])]), [[vue.resolveDirective("border-light"), {
        borderGradientSize: 160,
        bgColor: null
      }]]);
    };
  }
});
Card.install = (app) => {
  app.component(Card.name, Card);
};

module.exports = Card;
