'use strict';

var vue = require('vue');
var useMemo = require('../_util/hooks/useMemo.js');
var Group = require('./Group.js');

var Checkbox = vue.defineComponent({
  name: "x-checkbox",
  props: {
    prefixCls: {
      default: "x-checkbox"
    },
    checked: Boolean,
    disabled: Boolean,
    name: String,
    value: [String, Number],
    type: {
      default: "checkbox"
    }
  },
  emits: ["update:checked", "change"],
  setup(props, {
    emit,
    slots
  }) {
    const groupContext = vue.inject(Group.checkboxGroupContext);
    const disabledRef = useMemo(() => {
      var _a;
      return ((_a = groupContext == null ? void 0 : groupContext.disabledRef) == null ? void 0 : _a.value) || props.disabled;
    });
    const nameRef = useMemo(() => {
      var _a;
      return ((_a = groupContext == null ? void 0 : groupContext.nameRef) == null ? void 0 : _a.value) || props.name;
    });
    const state = vue.reactive({
      checked: props.checked,
      focused: false
    });
    vue.watch(() => props.checked, (checked) => {
      state.checked = checked;
    });
    const checkedRef = useMemo(() => {
      if (groupContext)
        return groupContext.valueRef.value.includes(props.value);
      return state.checked;
    });
    function onChange(e) {
      var _a;
      if (props.disabled)
        return;
      const val = e.target.checked;
      if (groupContext) {
        groupContext.toggleOption({
          label: (_a = slots.default) == null ? void 0 : _a.call(slots),
          value: props.value
        });
      } else {
        state.checked = val;
        emit("update:checked", val);
      }
      emit("change", val);
    }
    function onFocus() {
      state.focused = true;
    }
    function onBlur() {
      state.focused = false;
    }
    return () => {
      var _a;
      const {
        prefixCls
      } = props;
      const radioProps = {
        class: [prefixCls, {
          [`${prefixCls}-checked`]: checkedRef.value,
          [`${prefixCls}-focused`]: state.focused,
          [`${prefixCls}-disabled`]: disabledRef.value
        }]
      };
      const inputProps = {
        class: `${prefixCls}_input`,
        name: nameRef.value,
        value: props.value,
        type: props.type,
        checked: !!checkedRef.value,
        disabled: disabledRef.value,
        onFocus,
        onBlur,
        onChange
      };
      return vue.createVNode("label", radioProps, [vue.createVNode("input", inputProps, null), vue.createVNode("div", {
        "class": `${prefixCls}_dot`
      }, null), vue.createVNode("div", {
        "class": `${prefixCls}_label`
      }, [(_a = slots.default) == null ? void 0 : _a.call(slots)])]);
    };
  }
});

module.exports = Checkbox;
