'use strict';

var vue = require('vue');
var shared = require('@vue/shared');
var moment = require('moment');
var useMemo = require('../_util/hooks/useMemo.js');
var _ = require('../_util/_.js');
var utils = require('./utils.js');
var props = require('./props.js');
require('../trigger/index.js');
require('../icon/index.js');
require('../input/index.js');
require('../calendar/index.js');
var RangeCalendar = require('../calendar/RangeCalendar.js');
var Calendar = require('../calendar/Calendar.js');
var input = require('../input/input.js');
var Icon = require('../icon/Icon.js');
var Trigger = require('../trigger/Trigger.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var moment__default = /*#__PURE__*/_interopDefaultLegacy(moment);

function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !vue.isVNode(s);
}
const prefixCls = "x-date-picker";
const DatePicker = vue.defineComponent({
  name: prefixCls,
  props: props.PickerProps,
  emits: ["update:value", "change"],
  setup(props, {
    emit
  }) {
    const calendarRef = vue.ref();
    const inputRef = vue.ref();
    const visible = vue.ref(false);
    const valueRef = useMemo(() => utils.toMoment(props.value, props.valueFormat));
    const isRange = vue.computed(() => props.type.includes("range"));
    function onChange(date) {
      valueRef.value = isRange.value ? _.castArray(date).map((e) => e == null ? void 0 : e.clone()) : date == null ? void 0 : date.clone();
      if (shared.isArray(date)) {
        const val = props.valueFormat ? utils.formatDate(date, props.valueFormat) : date == null ? void 0 : date.map((e) => e.clone());
        emit("update:value", val);
        emit("change", val);
      } else {
        const val = props.valueFormat ? utils.formatDate(date, props.valueFormat) : date == null ? void 0 : date.clone();
        emit("update:value", val);
        emit("change", val);
      }
    }
    function onSelect(date) {
      console.log("date-picker");
      onChange(date);
      visible.value = false;
    }
    function onInput(str) {
      if (!str)
        return onChange(null);
      const date = utils.toMoment(str, props.format, true);
      if (_.castArray(date).every((e) => e.isValid())) {
        onChange(date);
      }
    }
    function onKeydown(e) {
      if (e.key === "Enter")
        visible.value = !visible.value;
    }
    function onBlur() {
      var _a;
      const temp = valueRef.value;
      valueRef.value = ((_a = _.castArray(valueRef.value)[0]) != null ? _a : moment__default["default"]()).clone().add(1, "days");
      vue.nextTick(() => valueRef.value = temp);
      const els = [calendarRef, inputRef].map((e) => e.value.$el).filter((e) => e);
      setTimeout(() => visible.value = els.some((e) => e == document.activeElement || e.contains(document.activeElement)), 0);
    }
    function renderCalendar() {
      const calendarProps = {
        ref: calendarRef,
        value: valueRef.value,
        disabledDate: props.disabledDate,
        tabindex: -1,
        onSelect,
        onBlur
      };
      return isRange.value ? vue.createVNode(RangeCalendar, calendarProps, null) : vue.createVNode(Calendar.DateCalendar, calendarProps, null);
    }
    function renderInput() {
      const {
        clearable,
        disabled,
        placeholder
      } = props;
      const inputProps = {
        ref: inputRef,
        value: utils.formatDate(valueRef.value, props.format),
        clearable,
        disabled,
        placeholder,
        pair: isRange.value,
        onInput,
        onKeydown
      };
      return vue.createVNode(input, vue.mergeProps(inputProps, {
        "class": `${prefixCls}-range`
      }), {
        suffix: () => vue.createVNode(Icon["default"], {
          "type": "calendar"
        }, null)
      });
    }
    return () => {
      let _slot;
      const {
        disabled
      } = props;
      const triggerProps = {
        prefixCls: `${prefixCls}-popup`,
        popup: renderCalendar(),
        popupVisible: visible.value,
        action: disabled ? "" : "click",
        popupAlign: {
          points: ["tl", "bl"]
        },
        onFocus: () => visible.value = true,
        onBlur
      };
      return vue.createVNode(Trigger, triggerProps, _isSlot(_slot = renderInput()) ? _slot : {
        default: () => [_slot]
      });
    };
  }
});
DatePicker.install = (app) => {
  app.component(DatePicker.name, DatePicker);
};

module.exports = DatePicker;
