'use strict';

var vue = require('vue');
var _interface = require('./interface.js');

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const formProps = {
  prefixCls: {
    default: "x-form"
  },
  model: Object,
  layout: {
    type: String,
    default: "horizontal"
  },
  labelWidth: Number,
  rules: Object
};
var Form = vue.defineComponent({
  name: "x-form",
  props: formProps,
  setup(props, {
    expose,
    slots
  }) {
    const formItems = {};
    function addItem(vm) {
      formItems[vm.props.prop] = vm;
    }
    function delItem(vm) {
      delete formItems[vm.props.prop];
    }
    function validate() {
      return __async(this, null, function* () {
        yield validateFields(Object.keys(formItems));
      });
    }
    function validateFields(fields) {
      return __async(this, null, function* () {
        const arr = fields.map((field) => formItems[field].validate());
        yield Promise.all(arr);
      });
    }
    function clearValidate() {
      Object.values(formItems).forEach((item) => item.clearValidate());
    }
    function resetFields() {
      Object.values(formItems).forEach((item) => item.resetField());
    }
    expose({
      validate,
      validateFields,
      clearValidate,
      resetFields
    });
    vue.provide(_interface.FormContextKey, {
      props,
      addItem,
      delItem
    });
    const onSubmit = (e) => e.preventDefault();
    return () => {
      const {
        prefixCls,
        layout
      } = props;
      const cls = [prefixCls, `${prefixCls}-${layout}`];
      return vue.createVNode("form", {
        "class": cls,
        "onSubmit": onSubmit
      }, __spreadValues({}, slots));
    };
  }
});

module.exports = Form;
