'use strict';

var vue = require('vue');
var index$1 = require('../_util/props-util/index.js');
var utils = require('./utils.js');
var index = require('../border-light/index.js');
require('../icon/index.js');
var Icon = require('../icon/Icon.js');

var MenuItemContent = vue.defineComponent({
  name: "x-menu-item-content",
  directives: {
    [index.name]: index
  },
  props: {
    node: Object,
    label: [String, Object],
    value: [String, Number],
    icon: [Object, Function],
    disabled: Boolean,
    showArrow: Boolean,
    active: Boolean,
    collapsed: Boolean
  },
  setup(props) {
    const {
      indentRef,
      menuContext
    } = utils.useMenu(props);
    return () => {
      const indent = indentRef.value;
      const prefixCls = "x-menu-item-content";
      const cls = {
        [prefixCls]: true,
        [`${prefixCls}-active`]: props.active,
        [`${prefixCls}-disabled`]: props.disabled,
        [`${prefixCls}-collapsed`]: props.collapsed
      };
      const iconNode = index$1.render(props.icon);
      let vnode = vue.createVNode("div", {
        "class": cls
      }, [iconNode && vue.cloneVNode(iconNode, {
        class: `${prefixCls}_icon`,
        style: `marginLeft:-${indent}px`,
        role: "none"
      }), vue.createVNode("span", {
        "class": `${prefixCls}_label`
      }, [props.label]), props.showArrow && vue.createVNode(Icon["default"], {
        "class": `${prefixCls}_arrow`,
        "type": "arrow-down"
      }, null)]);
      if (menuContext.props.borderLight) {
        const dir = vue.resolveDirective("border-light");
        const dirVal = {
          borderWidth: 1.2,
          borderGradientSize: 65,
          style: {
            opacity: 0.8
          }
        };
        vnode = vue.withDirectives(vnode, [[dir, dirVal]]);
      }
      return vnode;
    };
  }
});

module.exports = MenuItemContent;
