'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
var index = require('../_util/props-util/index.js');
require('../_util/hooks/index.js');
var normalizeOptions = require('../_util/normalizeOptions.js');
var Radio = require('./Radio.js');
var useMemo = require('../_util/hooks/useMemo.js');

const radioGroupContext = Symbol("radioGroupContext");
const prefixCls = "x-radio-group";
var RadioGroup = vue.defineComponent({
  name: prefixCls,
  props: {
    name: String,
    options: Array,
    value: String,
    vertical: Boolean,
    disabled: Boolean
  },
  emits: ["update:value", "change"],
  setup(props, {
    emit
  }) {
    const {
      proxy: self
    } = vue.getCurrentInstance();
    const val = useMemo(() => props.value);
    function onChange(e) {
      const val2 = e.target.value;
      val2.value = val2;
      emit("update:value", val2);
      emit("change", e);
    }
    vue.provide(radioGroupContext, {
      disabledRef: vue.toRef(props, "disabled"),
      nameRef: vue.toRef(props, "name"),
      valueRef: vue.toRef(val, "value"),
      onChange
    });
    return () => {
      var _a;
      let children;
      if ((_a = props.options) == null ? void 0 : _a.length) {
        children = normalizeOptions.mapOptions(props.options, (opt) => vue.createVNode(Radio["default"], {
          "key": opt.value,
          "checked": val.value == opt.value,
          "name": props.name,
          "value": opt.value,
          "disabled": opt.disabled || props.disabled,
          "style": opt.style
        }, {
          default: () => [opt.label]
        }));
      } else {
        children = index.filterEmpty(index.getSlot(self));
      }
      const cls = [prefixCls, props.vertical && `${prefixCls}-vertical`];
      return vue.createVNode("div", {
        "class": cls
      }, [children]);
    };
  }
});

exports["default"] = RadioGroup;
exports.radioGroupContext = radioGroupContext;
