'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
var index = require('../virtual-list/index.js');

const optionListProps = {
  prefixCls: String,
  options: Array,
  values: Set,
  virtualScroll: Boolean
};
var OptionList = vue.defineComponent({
  name: "x-option-list",
  props: optionListProps,
  slots: ["label"],
  emits: ["click", "select"],
  setup(props, {
    emit,
    slots
  }) {
    const {
      prefixCls
    } = props;
    const optionPrefixCls = `${prefixCls}-option`;
    function onClick(option) {
      emit("click", option);
      emit("select", option);
    }
    const genItem = (...args) => {
      var _a, _b;
      const {
        item: option,
        index
      } = args[0];
      let label = (_b = (_a = slots.default) == null ? void 0 : _a.call(slots, ...args)) != null ? _b : option.label;
      if (slots.label)
        label = slots.label({
          option,
          index
        });
      const selected = props.values.has(option.value);
      const clazz = [optionPrefixCls, {
        selected,
        [`${optionPrefixCls}-selected`]: selected,
        [`${optionPrefixCls}-disabled`]: option.disabled
      }];
      return vue.createVNode("div", vue.mergeProps({
        "class": clazz,
        "onClick": () => onClick(option)
      }, option), [vue.createVNode("div", {
        "class": `${optionPrefixCls}-content`
      }, [label])]);
    };
    return () => {
      const slots2 = {
        default: genItem
      };
      return vue.createVNode(index, {
        "virtual": props.virtualScroll,
        "items": props.options,
        "itemKey": "value"
      }, slots2);
    };
  }
});

exports["default"] = OptionList;
exports.optionListProps = optionListProps;
