'use strict';

var vue = require('vue');
var index = require('../_util/props-util/index.js');
require('../trigger/index.js');
var Trigger = require('../trigger/Trigger.js');

function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !vue.isVNode(s);
}
var SelectTrigger = vue.defineComponent({
  name: "SelectTrigger",
  props: {
    prefixCls: String,
    popup: Object,
    popupStyle: String,
    visible: Boolean,
    disabled: Boolean,
    filterable: Boolean
  },
  setup(props) {
    const {
      prefixCls
    } = props;
    const scrollRef = vue.ref();
    const popupRef = vue.ref();
    const alignRef = vue.ref();
    const targetRef = vue.ref();
    vue.provide("popupRef", popupRef);
    vue.provide("alignRef", alignRef);
    const getSelectedEl = () => {
      var _a;
      return ((_a = scrollRef.value) == null ? void 0 : _a.$el).querySelector(".selected");
    };
    const offset = vue.ref(0);
    const popupAlign = vue.computed(() => props.filterable ? {
      offset: [0, 0]
    } : {
      points: ["tl", "tl"],
      offset: [0, offset.value],
      overflow: {
        adjustX: 0,
        adjustY: 0
      },
      targetOffset: [0, 0]
    });
    function PopupTransition(_, {
      slots
    }) {
      return vue.createVNode(vue.Transition, {
        "appear": true,
        "onEnter": onPopupEnter
      }, slots);
    }
    function onPopupEnter() {
      var _a, _b;
      const scrollEl = (_a = scrollRef.value) == null ? void 0 : _a.$el;
      if (!scrollEl)
        return;
      const selectedEl = (_b = getSelectedEl()) != null ? _b : scrollEl.querySelector(`.${prefixCls}-option`);
      if (!selectedEl)
        return;
      const targetEl = index.getEL(targetRef.value);
      if (!targetEl)
        return;
      offset.value = 0;
      alignRef.value.forceAlign();
      offset.value = targetEl.getBoundingClientRect().top - selectedEl.getBoundingClientRect().top;
      requestAnimationFrame(() => {
        const scrollY = selectedEl.offsetTop;
        scrollEl.scroll(scrollEl.scrollLeft, scrollY);
      });
    }
    return (vm) => {
      let _slot;
      const listboxPreCls = `${prefixCls}-listbox`;
      const popup = vue.cloneVNode(props.popup, {
        ref: scrollRef
      }, true);
      return vue.createVNode(Trigger, {
        "popup": popup,
        "popupClass": listboxPreCls,
        "popupStyle": props.popupStyle,
        "popupVisible": props.visible,
        "popupPlacement": props.filterable ? "bottomLeft" : void 0,
        "popupTransition": PopupTransition,
        "popupTransitionName": listboxPreCls,
        "popupAlign": popupAlign
      }, _isSlot(_slot = vue.cloneVNode(index.getSlot(vm)[0], {
        ref: targetRef
      })) ? _slot : {
        default: () => [_slot]
      });
    };
  }
});

module.exports = SelectTrigger;
