'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var vue = require('vue');
var _interface = require('./interface.js');
var resizeobserver = require('../_util/Dom/resizeobserver.js');
var utils = require('./utils.js');
var shared = require('@vue/shared');

const setup = (el, binding, vnode) => {
  const props = vue.shallowReactive({
    el,
    binding,
    fragment: el.appendChild(document.createElement("div")),
    targets: vue.shallowReactive([]),
    vnode
  });
  el = null;
  binding = null;
  vnode = null;
  const unwatch1 = vue.watchPostEffect(() => {
    var _a;
    const { el: el2, binding: binding2, vnode: vnode2 } = props;
    if (utils.getLoading(binding2)) {
      utils.patchClass(el2, utils.getClass(binding2), utils.allClass(binding2));
      el2.style.cssText = shared.stringifyStyle(vue.normalizeStyle([el2.style.cssText, binding2.value.loadingStyle]));
    } else {
      el2.classList.remove(...utils.allClass(binding2).split(" "));
      el2.style.cssText = shared.stringifyStyle((_a = vnode2.props) == null ? void 0 : _a.style);
    }
  });
  const unwatch2 = vue.watchPostEffect(() => {
    const { binding: binding2, fragment } = props;
    const targets = props.targets, loading = utils.getLoading(binding2);
    if (loading)
      resizeobserver.observer(props.el);
    else
      resizeobserver.unobserve(props.el);
    if (loading) {
      targets.forEach((e) => e.el.classList.add(_interface.itemRefPrecls));
    } else {
      targets.forEach((e) => e.el.classList.remove(_interface.itemRefPrecls));
    }
    if (loading) {
      const items = targets.map((e) => {
        var _a, _b;
        return vue.h("div", {
          class: [props.binding.value.class, (_a = e.binding.value) == null ? void 0 : _a.class, _interface.itemPreCls],
          style: [props.binding.value.style, (_b = e.binding.value) == null ? void 0 : _b.style, e.state.style]
        });
      });
      vue.render(vue.h(vue.Fragment, items), fragment);
    } else {
      vue.render(null, fragment);
    }
  });
  props.unwatch = () => {
    unwatch1();
    unwatch2();
  };
  return props;
};
const skeletonList = [];
const Skeleton = {
  created(el, binding, vnode) {
    skeletonList.push(setup(el, binding, vnode));
  },
  beforeMount(el, binding, vnode) {
    const props = skeletonList.find((e) => e.el === el);
    props.binding = binding;
    props.vnode = vnode;
  },
  beforeUpdate(el, binding, vnode) {
    const props = skeletonList.find((e) => e.el === el);
    props.vnode = vnode;
    props.binding = binding;
  },
  beforeUnmount(el) {
    const i = skeletonList.findIndex((e) => e.el === el);
    if (i < 0)
      return;
    const props = skeletonList.splice(i, 1)[0];
    props.unwatch();
    props.fragment.parentNode.removeChild(props.fragment);
    resizeobserver.dispose(el);
  }
};

exports["default"] = Skeleton;
exports.skeletonList = skeletonList;
