'use strict';

var vue = require('vue');
var shared = require('@vue/shared');
var index = require('../_internal/display-directive/index.js');
require('../align/index.js');
var Align = require('../align/Align.js');

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
const popupProps = {
  prefixCls: String,
  visible: Boolean,
  getClassNameFromAlign: Function,
  getRootDomNode: Function,
  align: Object,
  displayDirective: String,
  getContainer: Function,
  transition: [Object, Function],
  transitionName: String,
  mask: Boolean,
  maskTransitionName: String,
  zIndex: Number,
  popupClass: [String, Object, Array],
  popupStyle: [String, Object, Array],
  disabledAlign: [Boolean, Object],
  disabledAlignInTransiting: [Boolean, Object],
  point: Object
};
var Popup = vue.defineComponent({
  name: "TriggerPopup",
  props: popupProps,
  inheritAttrs: false,
  emits: ["align"],
  setup(props, {
    attrs,
    slots,
    emit
  }) {
    const popupRef = vue.inject("popupRef", vue.ref());
    const alignRef = vue.inject("alignRef", vue.ref());
    const transiting = vue.ref(false);
    const disabledAlign = vue.computed(() => vue.unref(props.disabledAlign) || vue.unref(props.disabledAlignInTransiting) && transiting.value || !props.visible);
    function onEnter() {
      var _a;
      if (!popupRef.value)
        return;
      (_a = alignRef.value) == null ? void 0 : _a.forceAlign();
      ainBegin();
    }
    function ainBegin() {
      transiting.value = true;
    }
    function ainEnd() {
      transiting.value = false;
    }
    function onAlign(popupEl, align) {
      emit("align", ...arguments);
    }
    function getZIndexStyle() {
      const style = {};
      if (props.zIndex !== void 0) {
        style.zIndex = props.zIndex;
      }
      return style;
    }
    function _Transition(_, {
      slots: slots2
    }) {
      const transition = props.transition || vue.Transition;
      return vue.createVNode(transition, vue.mergeProps(_, {
        "appear": true,
        "onEnter": onEnter,
        "onAfterEnter": ainEnd,
        "onBeforeLeave": ainBegin,
        "onAfterLeave": ainEnd
      }), slots2);
    }
    function renderPopup() {
      const popupInnerProps = __spreadProps(__spreadValues({}, attrs), {
        style: shared.normalizeStyle([attrs.style, props.popupStyle, getZIndexStyle()]),
        class: shared.normalizeClass([attrs.class, props.prefixCls, props.popupClass])
      });
      const name = props.transitionName || props.prefixCls;
      const target = props.point || props.getRootDomNode;
      return vue.createVNode(Align, {
        "ref": alignRef,
        "target": target,
        "align": props.align,
        "disabled": disabledAlign,
        "onAlign": onAlign
      }, {
        default: () => [vue.createVNode(_Transition, {
          "name": name
        }, {
          default: () => [vue.createVNode(index, {
            "visible": props.visible,
            "type": "show"
          }, {
            default: () => {
              var _a;
              return [vue.createVNode("div", vue.mergeProps(popupInnerProps, {
                "ref": popupRef
              }), [(_a = slots.default) == null ? void 0 : _a.call(slots)])];
            }
          })]
        })]
      });
    }
    function renderMask() {
      if (props.mask) {
        const name = props.maskTransitionName || `${props.prefixCls}-mask`;
        return vue.createVNode(vue.Transition, {
          "appear": true,
          "name": name
        }, {
          default: () => [vue.withDirectives(vue.createVNode("div", {
            "class": `${props.prefixCls}-mask`,
            "style": getZIndexStyle()
          }, null), [[vue.vShow, props.visible]])]
        });
      }
    }
    return () => vue.createVNode("div", {
      "style": "position: absolute; top: 0; left: 0; width: 100%"
    }, [renderMask(), renderPopup()]);
  }
});

module.exports = Popup;
