'use strict';

var vue = require('vue');
var shared = require('@vue/shared');
var core = require('@vueuse/core');
var index = require('../_util/props-util/index.js');
var addEventListener = require('../_util/Dom/addEventListener.js');
var Popup = require('./Popup.js');
var placements = require('../tooltip/placements.js');
var index$1 = require('../_internal/lazy/index.js');
require('../_util/hooks/index.js');
var useMemo = require('../_util/hooks/useMemo.js');

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !vue.isVNode(s);
}
var Trigger = vue.defineComponent({
  name: "Trigger",
  inheritAttrs: false,
  props: {
    prefixCls: String,
    action: {
      type: String,
      default: ""
    },
    disabled: Boolean,
    // mask
    mask: Boolean,
    maskClosable: {
      type: Boolean,
      default: true
    },
    maskTransitionName: String,
    // popup
    popup: [String, Number, Object, Array],
    popupVisible: {
      type: Boolean,
      default: void 0
    },
    popupTransition: [Object, Function],
    popupTransitionName: {
      default: "zoom-ani"
    },
    popupStyle: [Array, Object, String],
    popupClass: [Array, Object, String],
    popupAlign: Object,
    popupPoint: Object,
    popupPlacement: {
      type: String
    },
    popupContainer: [Function, Object],
    builtinPlacements: {
      type: Object,
      default: placements.placements
    },
    // todo
    onPopupVisibleChange: Function,
    // todo
    afterPopupVisibleChange: Function,
    disabledAlign: [Boolean, Object],
    disabledAlignInTransiting: [Boolean, Object],
    zIndex: [String, Number],
    // target
    mouseEnterDelay: {
      default: 0
    },
    mouseLeaveDelay: {
      default: 150
    },
    focusDelay: {
      default: 0
    },
    blurDelay: {
      default: 0
    },
    // align
    alignPoint: Boolean
  },
  setup(props, {
    emit,
    attrs
  }) {
    const triggerRef = vue.ref();
    const {
      proxy: vm
    } = vue.getCurrentInstance();
    const state = vue.reactive({
      point: null
    });
    const visible = useMemo(() => props.popupVisible);
    vue.onMounted(() => {
      addOutsideHandler();
    });
    vue.onUpdated(() => {
      addOutsideHandler();
    });
    vue.onUnmounted(() => {
      clearDelayTimer();
      clearOutsideHandler();
    });
    function setPopupVisible(v, event) {
      if (props.disabled)
        return;
      if (visible.value != v) {
        if (props.popupVisible === void 0)
          visible.value = v;
        emit("popupVisibleChange", v);
      }
      setPoint(event);
    }
    function setPoint(point) {
      if (!props.alignPoint || !point)
        return;
      state.point = {
        pageX: point.pageX,
        pageY: point.pageY
      };
    }
    let delayTimer;
    function delaySetPopupVisible(visible2, delay, event) {
      clearDelayTimer();
      if (delay) {
        delayTimer = setTimeout(() => {
          setPopupVisible(visible2, event);
          clearDelayTimer();
        }, delay);
      } else {
        setPopupVisible(visible2, event);
      }
    }
    function clearDelayTimer() {
      clearTimeout(delayTimer);
    }
    function close() {
      setPopupVisible(false);
    }
    let clickOutsideHandler, ctxmOutsideHandler;
    function addOutsideHandler() {
      if (visible.value) {
        if (!clickOutsideHandler && props.action.includes("click")) {
          clickOutsideHandler = addEventListener(document, "mousedown", onDocumentClick);
        }
        if (!ctxmOutsideHandler && props.action.includes("contextmenu")) {
          clickOutsideHandler = addEventListener(document, "mousedown", onDocumentClick);
        }
      } else {
        clearOutsideHandler();
      }
    }
    function clearOutsideHandler() {
      clickOutsideHandler == null ? void 0 : clickOutsideHandler.remove();
      clickOutsideHandler = null;
      ctxmOutsideHandler == null ? void 0 : ctxmOutsideHandler.remove();
      ctxmOutsideHandler = null;
    }
    function onDocumentClick(event) {
      if (popupMousedowned)
        return;
      if (props.mask && !props.maskClosable)
        return;
      const root = index.findDOMNode(vm);
      if (!root.contains(event.target)) {
        close();
      }
    }
    let popupMousedowned = false;
    const align = vue.computed(() => {
      var _a;
      return __spreadValues(__spreadValues({}, (_a = props.builtinPlacements) == null ? void 0 : _a[props.popupPlacement]), vue.unref(props.popupAlign));
    });
    function renderPopup() {
      let _slot;
      const mouseProps = {};
      if (props.action.includes("hover")) {
        mouseProps.onMouseenter = clearDelayTimer;
        mouseProps.onMouseleave = onPopupMouseleave;
      }
      mouseProps.onMousedown = onPopupMousedown;
      mouseProps.onMouseup = onPopupMouseup;
      const {
        prefixCls,
        popupPlacement
      } = props;
      const _visible = visible.value && !props.disabled;
      let placementCls = popupPlacement ? `${prefixCls}-placement-${popupPlacement}` : "";
      const popupPorps = __spreadValues({
        prefixCls,
        visible: _visible,
        point: props.alignPoint ? props.popupPoint || state.point : null,
        action: props.action,
        align,
        getRootDomNode,
        mask: props.mask,
        maskTransitionName: props.maskTransitionName,
        // getContainer,
        popupStyle: props.popupStyle,
        popupClass: vue.normalizeClass([props.popupClass, placementCls]),
        disabledAlign: props.disabledAlign,
        disabledAlignInTransiting: props.disabledAlignInTransiting,
        transition: props.popupTransition,
        transitionName: props.popupTransitionName,
        onAlign: attrs.onPopupAlign || shared.NOOP,
        zIndex: props.zIndex
      }, mouseProps);
      return vue.createVNode(index$1, {
        "visible": _visible
      }, {
        default: () => [vue.createVNode(Popup, popupPorps, _isSlot(_slot = index.getComponent(vm, "popup")) ? _slot : {
          default: () => [_slot]
        })]
      });
    }
    function popupContainer() {
      var _a;
      return (_a = core.unrefElement(props.popupContainer)) != null ? _a : document.body;
    }
    let focused = false;
    function onClick(e) {
      setPopupVisible(!visible.value, e);
    }
    function onClickHide() {
      setPopupVisible(false);
    }
    function onContextmenu(e) {
      e.preventDefault();
      setPopupVisible(true, e);
    }
    function onMouseenter(e) {
      delaySetPopupVisible(true, props.mouseEnterDelay, e);
    }
    function onFocus(e) {
      focused = true;
      delaySetPopupVisible(true, props.focusDelay, e);
    }
    function onBlur(e) {
      focused = false;
      delaySetPopupVisible(false, props.blurDelay, e);
    }
    function onPopupMouseleave(e) {
      if (props.action.includes("focus") && focused)
        return;
      delaySetPopupVisible(false, props.mouseLeaveDelay, e);
    }
    function onPopupMousedown() {
      popupMousedowned = true;
    }
    function onPopupMouseup() {
      popupMousedowned = false;
    }
    function getRootDomNode() {
      return index.findDOMNode(triggerRef.value);
    }
    return (vm2) => {
      let _slot2;
      let child = index.filterEmpty(index.getSlot(vm2))[0];
      child = index.isValidElement(child) ? child : vue.createVNode("span", null, [child]);
      const newProps = {
        ref: triggerRef
      };
      if (props.action.includes("click")) {
        newProps.onClick = onClick;
      }
      if (props.action.includes("contextmenu")) {
        newProps.onContextmenu = onContextmenu;
        newProps.onClick = onClickHide;
      }
      if (props.action.includes("hover")) {
        newProps.onMouseenter = onMouseenter;
        newProps.onMouseleave = onPopupMouseleave;
      }
      if (props.action.includes("focus")) {
        newProps.onFocus = onFocus;
        newProps.onBlur = onBlur;
      }
      const trigger = vue.cloneVNode(child, vue.mergeProps(newProps, attrs), true);
      const popup = triggerRef.value ? vue.createVNode(vue.Teleport, {
        "to": popupContainer()
      }, _isSlot(_slot2 = renderPopup()) ? _slot2 : {
        default: () => [_slot2]
      }) : void 0;
      return [trigger, popup];
    };
  }
});

module.exports = Trigger;
