import { watchPostEffect } from 'vue';

function useCssVars(el, vars) {
  const setVars = () => setVarsOnNode(el, vars);
  const unwatch = watchPostEffect(setVars);
  const ob = new MutationObserver(setVars);
  ob.observe(el.parentNode, { childList: true });
  return function unmount() {
    ob.disconnect();
    unwatch();
  };
}
function setVarsOnNode(el, vars) {
  if (el.nodeType === 1) {
    const style = el.style;
    for (const key in vars) {
      style.setProperty(`--${key}`, vars[key].toString());
    }
  }
  return () => removeVarsOnNode(el, Object.keys(vars));
}
function removeVarsOnNode(el, vars) {
  if (el.nodeType === 1) {
    const style = el.style;
    vars.forEach((key) => style.removeProperty(`--${key}`));
  }
}

export { removeVarsOnNode, setVarsOnNode, useCssVars };
