import { unref, onMounted } from 'vue';
import { findDOMNode } from '../props-util/index.js';

function getPosition(e) {
  if ("touches" in e) {
    return [e.touches[0].pageX, e.touches[0].pageY];
  } else {
    return [e.pageX, e.pageY];
  }
}
function useDraggable(el, options) {
  let startX = 0, startY = 0, startPageX = 0, startPageY = 0;
  function onMouseDown(e) {
    var _a;
    e.preventDefault();
    const style = getComputedStyle(unref(el));
    startX = style.left ? parseInt(style.left) : 0;
    startY = style.top ? parseInt(style.top) : 0;
    [startPageX, startPageY] = getPosition(e);
    window.addEventListener("mousemove", onMouseMove);
    window.addEventListener("mouseup", onMouseUp);
    (_a = options.onDragStart) == null ? void 0 : _a.call(options, e);
  }
  function onMouseMove(e) {
    var _a;
    e.preventDefault();
    const _el = unref(el);
    const [pageX, pageY] = getPosition(e);
    const x = startX + pageX - startPageX, y = startY + pageY - startPageY;
    setPosition(x, y);
    _el.style.zIndex = "1";
    (_a = options.onDragging) == null ? void 0 : _a.call(options, e);
  }
  function onMouseUp(e) {
    var _a;
    window.removeEventListener("mousemove", onMouseMove);
    window.removeEventListener("mouseup", onMouseUp);
    (_a = options.onDragEnd) == null ? void 0 : _a.call(options, e);
  }
  function setPosition(x, y) {
    const _el = unref(el);
    _el.style.left = x + "px";
    _el.style.top = y + "px";
    if (options.x)
      options.x.value = x;
    if (options.y)
      options.y.value = y;
  }
  let oldTarget;
  function enable(bool) {
    if (bool) {
      enable(false);
      oldTarget = findDOMNode(unref(options.dragTarget) || unref(el));
      oldTarget == null ? void 0 : oldTarget.addEventListener("mousedown", onMouseDown);
    } else {
      oldTarget == null ? void 0 : oldTarget.removeEventListener("mousedown", onMouseDown);
      oldTarget = null;
      onMouseUp();
    }
  }
  function reset() {
    const _el = unref(el);
    _el.style.left = "";
    _el.style.top = "";
    _el.style.zIndex = "";
  }
  onMounted(() => {
    enable(true);
  });
  return {
    enable,
    reset
  };
}

export { useDraggable as default };
