import { Comment, Fragment, Text, isVNode, createTextVNode, cloneVNode } from 'vue';
import { isOn, camelize, hyphenate } from '@vue/shared';
import isValid from '../isValid.js';

const splitAttrs = (attrs) => {
  const allAttrs = Object.keys(attrs);
  const eventAttrs = {};
  const onEvents = {};
  const extraAttrs = {};
  for (let i = 0, l = allAttrs.length; i < l; i++) {
    const key = allAttrs[i];
    if (isOn(key)) {
      eventAttrs[key[2].toLowerCase() + key.slice(3)] = attrs[key];
      onEvents[key] = attrs[key];
    } else {
      extraAttrs[key] = attrs[key];
    }
  }
  return { onEvents, events: eventAttrs, extraAttrs };
};
const getVm = (ins) => (ins == null ? void 0 : ins.proxy) || ins;
const getIns = (vm) => (vm == null ? void 0 : vm.$) || vm;
const getEL = (ins) => {
  var _a;
  return ins instanceof Node ? ins : (_a = getVm(ins)) == null ? void 0 : _a.$el;
};
function getProp(props, prop) {
  return props[camelize(prop)] || props[hyphenate(prop)];
}
function hasProp(ins, prop) {
  const props = getIns(ins).vnode.props;
  return camelize(prop) in props || hyphenate(prop) in props;
}
const findDOMNode = (instance) => {
  let node = getEL(instance) || instance;
  while (node && !node.tagName)
    node = node.nextSibling;
  return node;
};
function isEmptyElement(c) {
  return c.type === Comment || c.type === Fragment && c.children.length === 0 || c.type === Text && c.children.trim() === "";
}
function filterEmpty(children = []) {
  const res = [];
  children.forEach((child) => {
    if (Array.isArray(child)) {
      res.push(...child);
    } else if (child.type === Fragment) {
      res.push(...child.children);
    } else {
      res.push(child);
    }
  });
  return res.filter((c) => !isEmptyElement(c));
}
const flattenChildren = (children, filterEmpty2 = true) => {
  const temp = Array.isArray(children) ? children : [children];
  const res = [];
  for (let i = 0; i < temp.length; i++) {
    const child = temp[i];
    if (Array.isArray(child)) {
      res.push(...flattenChildren(child, filterEmpty2));
    } else if (isVNode(child)) {
      if (child.type === Fragment) {
        res.push(...flattenChildren(child.children, filterEmpty2));
      } else if (filterEmpty2 && !isEmptyElement(child)) {
        res.push(child);
      } else if (!filterEmpty2) {
        res.push(child);
      }
    } else if (isValid(child)) {
      if (!isEmptyElement)
        res.push(child);
    }
  }
  return res;
};
const getSlot = (self, name = "default", options = {}) => {
  var _a, _b;
  let res;
  if (typeof self === "function") {
    res = self(options);
  } else {
    res = (_b = (_a = getIns(self).slots || self)[name]) == null ? void 0 : _b.call(_a, options);
  }
  return flattenChildren(res);
};
const getComponent = (ins, prop = "default", opts = ins) => {
  let props = getIns(ins).props || ins, slots = getIns(ins).slots;
  const val = props[prop];
  return render(val != null && typeof val !== "boolean" ? val : slots == null ? void 0 : slots[prop], opts);
};
const render = (content, ...args) => {
  if (typeof content === "function") {
    return content(...args);
  } else if (typeof content === "object" && content) {
    return content;
  } else if (isValid(content)) {
    return createTextVNode(String(content));
  } else {
    return void 0;
  }
};
function _cloneVNode(vnode, extraProps, mergeRef) {
  if (!vnode)
    return void 0;
  return cloneVNode(vnode, extraProps, mergeRef);
}
function isValidElement(element) {
  return isVNode(element) && typeof element.type !== "symbol";
}
function hasElemnt(element) {
  if (element == null) {
    return false;
  } else if (Array.isArray(element)) {
    return element.some(hasElemnt);
  } else {
    if (isVNode(element)) {
      if (typeof element.type === "symbol") {
        return hasElemnt(element.children);
      } else {
        return true;
      }
    }
    return typeof element === "string" || typeof element === "number";
  }
}

export { _cloneVNode, filterEmpty, findDOMNode, flattenChildren, getComponent, getEL, getProp, getSlot, hasElemnt, hasProp, isEmptyElement, isValidElement, render, splitAttrs };
