import { defineComponent, ref, watch, watchPostEffect, provide, computed, onMounted, onBeforeUnmount } from 'vue';
import { unrefElement } from '@vueuse/core';
import { getScrollParent } from '../_util/Dom/getScrollParent.js';
import '../_util/hooks/index.js';
import { throttle } from '../_util/_.js';
import { AnchorContextKey } from './context.js';
import { scroller } from './utils.js';
import useNamespace from '../_util/hooks/useNameSpace2.js';

const preCls = "x-anchor";
const anchorProps = {
  value: String,
  type: { type: String, default: "rail" },
  showBackground: { type: Boolean, default: true },
  offset: { type: Number, default: 0 },
  container: [String, Object, Function],
  scrollDuration: { type: Number, default: 350 }
};
var _sfc_main = defineComponent({
  name: preCls,
  props: anchorProps,
  emits: ["change"],
  setup(props, { emit }) {
    const ns = useNamespace(preCls);
    const anchorRef = ref();
    const barRef = ref();
    const activeRef = ref("");
    const links = [];
    watch(activeRef, (v) => emit("change", v));
    watchPostEffect(() => {
      var _a;
      if (!barRef.value)
        return;
      const el = (_a = links.find((e) => e.href == activeRef.value)) == null ? void 0 : _a.el;
      if (el) {
        const boundTop = anchorRef.value.getBoundingClientRect().top;
        const rect = el.getBoundingClientRect();
        const h = rect.height;
        barRef.value.style.height = h + "px";
        barRef.value.style.width = "100%";
        barRef.value.style.top = rect.top - boundTop + (h >> 1) + "px";
      } else {
        barRef.value.style.height = "0px";
        barRef.value.style.width = "0px";
      }
    });
    provide(AnchorContextKey, {
      value: activeRef,
      links,
      setActive,
      props
    });
    const container = computed(() => {
      return props.container ? typeof props.container == "string" ? document.querySelector(props.container) : unrefElement(props.container) : getScrollParent(anchorRef.value);
    });
    const onScroll = throttle(computeCurrentLink, 128);
    const addListener = () => {
      var _a;
      (_a = container.value) == null ? void 0 : _a.addEventListener("scroll", onScroll, { passive: true });
    };
    const delListener = () => {
      container.value.removeEventListener("scroll", onScroll);
      onScroll.cancel();
    };
    onMounted(() => {
      watch(
        container,
        (val, old) => {
          old == null ? void 0 : old.removeEventListener("scroll", onScroll);
          addListener();
        },
        { immediate: true, flush: "post" }
      );
    });
    onBeforeUnmount(() => {
      delListener();
    });
    function getBoundTop() {
      return container.value.nodeType != 9 ? container.value.getBoundingClientRect().top : 0;
    }
    const scrollEl = computed(() => container.value.nodeType == 9 ? document.documentElement : container.value);
    const scroller$1 = scroller(scrollEl, 350, delListener, addListener);
    function setActive(href) {
      activeRef.value = href;
      const el = container.value.querySelector(href);
      if (!el)
        return;
      const rect = el.getBoundingClientRect();
      const y = scrollEl.value.scrollTop + (rect.top - getBoundTop()) - props.offset + 1;
      scroller$1.scroll({ y });
      location.hash = encodeURIComponent(href.replace(/^#/, ""));
    }
    function computeCurrentLink() {
      if (scroller$1.transiting)
        return;
      const { offset } = props;
      const boundTop = getBoundTop();
      const infos = links.map((e) => {
        const el = container.value.querySelector(e.href);
        return {
          href: e.href,
          rect: el == null ? void 0 : el.getBoundingClientRect()
        };
      }).filter((e) => e.rect).sort((a, b) => a.rect.top - b.rect.top);
      let info;
      for (let i = 0; i < infos.length; i++) {
        if (infos[i].rect.top <= boundTop + offset)
          info = infos[i];
        else
          break;
      }
      activeRef.value = info == null ? void 0 : info.href;
    }
    return {
      ns,
      anchorRef,
      barRef
    };
  }
});

export { _sfc_main as default };
