import { defineComponent, ref, onMounted, computed, withDirectives, createVNode, resolveDirective } from 'vue';
import '../_util/hooks/index.js';
import Sunken from '../sunken/index.js';
import '../icon/index.js';
import useNamespace from '../_util/hooks/useNameSpace2.js';
import Icon from '../icon/Icon.js';

const Button = defineComponent({
  name: "x-button",
  props: {
    type: {
      type: String,
      default: "default"
    },
    tag: {
      type: String,
      default: "button"
    },
    dashed: Boolean,
    circle: Boolean,
    round: Boolean,
    ghost: Boolean,
    link: Boolean,
    block: Boolean,
    disabled: Boolean,
    href: String,
    target: String,
    autofocus: Boolean,
    loading: Boolean
  },
  emits: ["click"],
  directives: {
    [Sunken.name]: Sunken
  },
  setup(props, {
    slots,
    emit
  }) {
    const ns = useNamespace("btn");
    const btnRef = ref();
    if (props.autofocus) {
      onMounted(() => {
        btnRef.value.focus();
      });
    }
    const disabledRef = computed(() => props.disabled || props.loading);
    function onClick(e) {
      if (disabledRef.value) {
        e.preventDefault();
        return;
      }
      emit("click", e);
    }
    return () => {
      const {
        tag,
        loading
      } = props;
      const disabled = disabledRef.value;
      const buttonProps = {
        ref: btnRef,
        class: {
          [ns.b()]: true,
          [ns.m(props.type)]: true,
          [ns.m("dashed")]: props.dashed,
          [ns.m("circle")]: props.circle,
          [ns.m("round")]: props.round,
          [ns.m("ghost")]: props.ghost || props.dashed,
          [ns.m("link")]: props.link,
          [ns.m("block")]: props.block,
          [ns.m("disabled")]: props.disabled,
          [ns.m("only-icon")]: !slots.default
        },
        disabled,
        onClick
      };
      return withDirectives(createVNode(tag, buttonProps, {
        default: () => {
          var _a;
          return [loading && createVNode(Icon, {
            "type": "loading",
            "spin": true,
            "class": ns.e("icon")
          }, null), (_a = slots.default) == null ? void 0 : _a.call(slots)];
        }
      }), [[resolveDirective("sunken")]]);
    };
  }
});
Button.install = (app) => {
  app.component(Button.name, Button);
};

export { Button as default };
