import { defineComponent, ref, isVNode, createVNode, cloneVNode, withDirectives } from 'vue';
import { castArray } from '../_util/_.js';
import BorderLight from '../border-light/index.js';
import Scroll from './Scroll.js';
import { filterEmpty } from '../_util/props-util/index.js';
import { isArray } from '@vue/shared';

const props = {
  prefixCls: String,
  start: Object,
  modelValue: [Object, Array],
  rangeValue: Array,
  cols: Number,
  limit: Array,
  //
  disabled: Function,
  add: Function,
  diff: Function,
  isBefore: Function,
  isAfter: Function,
  isSame: Function,
  group: Function,
  isBeforeGroup: Function,
  isAfterGroup: Function,
  addGroup: Function,
  //
  cell: Function,
  onSelect: Function,
  onHover: Function,
  onGroupChange: Function
};
var Grid = defineComponent({
  props,
  emits: ["update:modelValue"],
  setup(props2, {
    slots,
    emit,
    expose
  }) {
    var _a;
    function onSelect(date) {
      var _a2;
      emit("update:modelValue", date);
      (_a2 = props2.onSelect) == null ? void 0 : _a2.call(props2, date);
    }
    const scrollEl = ref();
    const group = ref(props2.group((_a = castArray(props2.modelValue)[0]) != null ? _a : props2.add(props2.start, props2.cols - 1)));
    function onScroll(e) {
      var _a2;
      const amount = scrollEl.value.calc(e.y + 120).startI * props2.cols;
      group.value = props2.group(props2.add(props2.start, amount));
      (_a2 = props2.onGroupChange) == null ? void 0 : _a2.call(props2, group.value);
    }
    function scrollIndex(index, t) {
      scrollEl.value.scrollIndex(Math.floor(index / props2.cols), t);
    }
    function scroll(date, t = 200) {
      const diff = props2.diff(props2.group(date), props2.start);
      scrollIndex(diff, t);
    }
    const prev = () => scroll(props2.addGroup(group.value, -1));
    const next = () => scroll(props2.addGroup(group.value, 1));
    expose({
      prev,
      next,
      scroll,
      scrollIndex
    });
    function genRow({
      index
    }) {
      var _a2, _b, _c, _d, _e, _f, _g;
      const {
        prefixCls,
        cols,
        rangeValue: range
      } = props2;
      const startI = index * cols;
      const endI = startI + cols;
      const val = castArray((_a2 = props2.modelValue) != null ? _a2 : props2.rangeValue);
      const cells = Array(cols);
      for (let i = startI; i < endI; i++) {
        const date = props2.add(props2.start, i);
        const disabled = (_b = props2.disabled) == null ? void 0 : _b.call(props2, date);
        const isPrevGroup = (_c = props2.isBeforeGroup) == null ? void 0 : _c.call(props2, date, group.value);
        const isNextGroup = (_d = props2.isAfterGroup) == null ? void 0 : _d.call(props2, date, group.value);
        const selected = val.some((e) => {
          var _a3;
          return (_a3 = props2.isSame) == null ? void 0 : _a3.call(props2, e, date);
        });
        const inRange = (range == null ? void 0 : range.length) == 2 && ((_e = props2.isAfter) == null ? void 0 : _e.call(props2, date, range[0])) && ((_f = props2.isBefore) == null ? void 0 : _f.call(props2, date, range[1]));
        const cls = {
          [`${prefixCls}_cell`]: true,
          [`${prefixCls}_cell-disabled`]: disabled,
          [`${prefixCls}_cell-selected`]: selected,
          [`${prefixCls}_cell-in-range`]: inRange,
          [`${prefixCls}_cell-prev`]: isPrevGroup,
          [`${prefixCls}_cell-next`]: isNextGroup,
          [`${prefixCls}_cell-this`]: (_g = props2.isSame) == null ? void 0 : _g.call(props2, date, new Date())
        };
        let child = (props2.cell || slots.default)(date);
        if (!isVNode(child)) {
          if (isArray(child)) {
            child = filterEmpty(child);
            child = child.length == 1 ? child[0] : createVNode("div", null, [child]);
          } else {
            const _child = function() {
              return child;
            }();
            child = createVNode("div", null, [child]);
          }
        }
        if (!isVNode(child)) {
          throw new Error("child in genRow is not a VNode");
        }
        const cellProps = {
          key: date.toString(),
          role: "gridcell",
          "aria-selected": selected,
          title: date.toString(),
          class: cls,
          onClick: () => onSelect(date),
          onMouseenter: () => {
            var _a3;
            return (_a3 = props2.onHover) == null ? void 0 : _a3.call(props2, date);
          }
        };
        child = cloneVNode(child, cellProps);
        child = withDirectives(child, [[BorderLight, {
          borderGradientSize: 50,
          bgColor: "transparent",
          borderColor: "#80808060"
        }]]);
        cells[i - startI] = child;
      }
      return cells;
    }
    return () => {
      const style = `display: grid; grid-template-columns: repeat(${props2.cols}, 1fr)`;
      return createVNode(Scroll, {
        "ref": scrollEl,
        "style": style,
        "onScroll": onScroll
      }, {
        default: genRow
      });
    };
  }
});

export { Grid as default };
