import { defineComponent, createVNode, cloneVNode, withDirectives, resolveDirective } from 'vue';
import BorderLight from '../border-light/index.js';
import '../icon/index.js';
import { getComponent, flattenChildren, hasElemnt } from '../_util/props-util/index.js';
import Icon from '../icon/Icon.js';

const Card = defineComponent({
  name: "x-card",
  props: {
    prefixCls: {
      default: "x-card"
    },
    title: String,
    icon: Object,
    bordered: {
      default: true
    },
    hoverable: Boolean,
    closable: Boolean,
    contentClass: [Object, String, Array],
    contentStyle: [Object, String],
    footerClass: [Object, String, Array],
    footerStyle: [Object, String],
    headerClass: [Object, String, Array],
    headerStyle: [Object, String],
    size: String
  },
  slots: ["title", "cover", "header", "header-extra", "default", "footer"],
  emits: ["close"],
  directives: {
    [BorderLight.name]: BorderLight
  },
  setup(props, {
    emit,
    slots
  }) {
    function close(e) {
      e.stopPropagation();
      e.preventDefault();
      emit("close");
    }
    return (vm) => {
      var _a, _b, _c, _d, _e;
      const {
        prefixCls,
        size,
        closable,
        contentClass,
        footerClass,
        headerClass,
        icon
      } = props;
      const cls = [prefixCls, {
        [`${prefixCls}-bordered`]: props.bordered,
        [`${prefixCls}-hoverable`]: props.hoverable,
        [`${prefixCls}-${size}`]: size
      }];
      const coverNode = (_a = slots.cover) == null ? void 0 : _a.call(slots);
      const titleNode = getComponent(vm, "title");
      const closeNode = closable ? createVNode(Icon, {
        "class": `${prefixCls}_close`,
        "type": "cross",
        "onClick": close
      }, null) : void 0;
      const contentNode = (_b = slots.default) == null ? void 0 : _b.call(slots);
      const footerNode = (_c = slots.footer) == null ? void 0 : _c.call(slots);
      const iconNode = flattenChildren(getComponent(vm, "icon"))[0];
      const headerExtraNode = (_d = slots["header-extra"]) == null ? void 0 : _d.call(slots);
      const headerNode = slots.header ? createVNode("div", {
        "class": [`${prefixCls}_header`, headerClass]
      }, [(_e = slots.header) == null ? void 0 : _e.call(slots)]) : createVNode("div", {
        "class": `${prefixCls}_header`,
        "style": props.headerStyle
      }, [hasElemnt(iconNode) && cloneVNode(iconNode, {
        class: `${prefixCls}_icon`
      }), createVNode("div", {
        "class": `${prefixCls}_title`
      }, [titleNode]), headerExtraNode && createVNode("div", {
        "class": `${prefixCls}_header_extra`
      }, [headerExtraNode]), closeNode]);
      return withDirectives(createVNode("div", {
        "class": cls
      }, [hasElemnt(coverNode) && createVNode("div", {
        "class": `${prefixCls}_cover`
      }, [coverNode]), headerNode, hasElemnt(contentNode) && createVNode("div", {
        "class": [`${prefixCls}_content`, contentClass],
        "style": props.contentStyle
      }, [contentNode]), hasElemnt(footerNode) && createVNode("div", {
        "class": [`${prefixCls}_footer`, footerClass],
        "style": props.footerStyle
      }, [footerNode])]), [[resolveDirective("border-light"), {
        borderGradientSize: 160,
        bgColor: null
      }]]);
    };
  }
});
Card.install = (app) => {
  app.component(Card.name, Card);
};

export { Card as default };
