import { defineComponent, getCurrentInstance, provide, toRef, createVNode } from 'vue';
import { getSlot } from '../_util/props-util/index.js';
import '../_util/hooks/index.js';
import { mapOptions } from '../_util/normalizeOptions.js';
import Checkbox from './Checkbox.js';
import useMemo from '../_util/hooks/useMemo.js';

const checkboxGroupContext = Symbol("checkboxGroupContext");
const prefixCls = "x-checkbox-group";
var Group = defineComponent({
  name: prefixCls,
  props: {
    name: String,
    options: Array,
    value: Array,
    vertical: Boolean,
    disabled: Boolean
  },
  emits: ["update:value", "change"],
  setup(props, {
    emit
  }) {
    const {
      proxy: vm
    } = getCurrentInstance();
    const val = useMemo(() => props.value);
    function toggleOption(opt) {
      const value = [...val.value];
      const i = value.indexOf(opt.value);
      if (i > -1) {
        value.splice(i, 1);
      } else {
        value.push(opt.value);
      }
      emit("update:value", value);
      emit("change", value);
    }
    provide(checkboxGroupContext, {
      disabledRef: toRef(props, "disabled"),
      nameRef: toRef(props, "name"),
      valueRef: toRef(val, "value"),
      toggleOption
    });
    return () => {
      var _a;
      let children;
      if ((_a = props.options) == null ? void 0 : _a.length) {
        children = mapOptions(props.options, (opt) => createVNode(Checkbox, {
          "key": opt.value,
          "checked": val.value.indexOf(opt.value) != -1,
          "name": props.name,
          "value": opt.value,
          "disabled": opt.disabled || props.disabled,
          "style": opt.style
        }, {
          default: () => [opt.label]
        }));
      } else {
        children = getSlot(vm);
      }
      const cls = [prefixCls, props.vertical && `${prefixCls}-vertical`];
      return createVNode("div", {
        "class": cls
      }, [children]);
    };
  }
});

export { checkboxGroupContext, Group as default };
