import { defineComponent, getCurrentInstance, createVNode, cloneVNode } from 'vue';
import { normalizeClass } from '@vue/shared';
import { hasProp, getComponent, isValidElement } from '../_util/props-util/index.js';
import useMemo from '../_util/hooks/useMemo.js';
import '../trigger/index.js';
import { placements } from './placements.js';
import tooltipProps from '../tooltip/tooltipProps.js';
import '../expand-transition/index.js';
import ExpandTransition from '../expand-transition/expand-transition.js';
import Trigger from '../trigger/Trigger.js';

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
const prefixCls = "x-dropdown";
const Dropdown = defineComponent({
  name: prefixCls,
  props: __spreadProps(__spreadValues({}, tooltipProps), {
    placement: {
      type: String,
      default: "bottomLeft"
    },
    popup: [String, Number, Object, Array],
    hasBg: {
      type: Boolean,
      default: true
    }
  }),
  emits: ["update:visible", "visibleChange"],
  slots: ["popup"],
  setup(props, {
    emit,
    slots
  }) {
    const {
      proxy: vm
    } = getCurrentInstance();
    const visibleRef = useMemo(() => props.visible);
    function setVisible(visible) {
      if (!hasProp(vm, "visible")) {
        visibleRef.value = visible;
      }
      emit("update:visible", visible);
      emit("visibleChange", visible);
    }
    let popupPoint;
    let onAfterLeave;
    function onContextmenu(e) {
      if (!props.trigger.includes("contextmenu"))
        return;
      if (visibleRef.value) {
        visibleRef.value = false;
        onAfterLeave = () => {
          console.log("() => {}");
          popupPoint = {
            pageX: e.pageX,
            pageY: e.pageY
          };
          visibleRef.value = true;
          onAfterLeave = null;
        };
      } else {
        popupPoint = {
          pageX: e.pageX,
          pageY: e.pageY
        };
        visibleRef.value = true;
      }
    }
    function onPopupVisibleChange(visible) {
      if (!visible)
        setVisible(visible);
      else if (props.trigger.includes("contextmenu"))
        return;
      else
        setVisible(visible);
    }
    function onClick() {
      onPopupVisibleChange(false);
    }
    function preventContextmenu(e) {
      if (!props.trigger.includes("contextmenu"))
        return;
      e.preventDefault();
    }
    const h = (h2) => h2 * 0.5;
    function PopupTransition(_, {
      slots: slots2
    }) {
      return createVNode(ExpandTransition, {
        "appear": true,
        "name": prefixCls,
        "height": h,
        "onAfterLeave": onAfterLeave
      }, slots2);
    }
    function getPopup() {
      const popupProps = {
        selectable: false,
        onClick,
        onContextmenu: preventContextmenu
      };
      let popup = getComponent(vm, "popup");
      popup = Array.isArray(popup) ? popup[0] : popup;
      popup = isValidElement(popup) ? popup : createVNode("div", null, [popup]);
      popup = cloneVNode(popup, popupProps);
      return createVNode("div", null, [popup]);
    }
    return () => {
      const {
        trigger,
        placement,
        builtinPlacements,
        disabled
      } = props;
      const popupClass = normalizeClass([`${prefixCls} ${prefixCls}-placement-${props.placement}`, props.hasBg && `${prefixCls}-has-bg`, props.placement.includes("bottom") && `${prefixCls}-bottom`, props.popupClass]);
      const triggerProps = {
        prefixCls,
        action: disabled ? "" : trigger,
        popup: getPopup(),
        popupVisible: visibleRef.value,
        popupClass,
        popupTransition: PopupTransition,
        popupAlign: props.align,
        popupPoint,
        popupStyle: props.popupStyle,
        disabledAlignInTransiting: true,
        popupPlacement: placement,
        builtinPlacements: builtinPlacements || placements,
        getPopupContainer: props.getPopupContainer,
        onPopupVisibleChange,
        mouseEnterDelay: props.mouseEnterDelay,
        mouseLeaveDelay: props.mouseLeaveDelay,
        alignPoint: trigger.includes("contextmenu"),
        onContextmenu
      };
      return createVNode(Trigger, triggerProps, slots);
    };
  }
});
Dropdown.install = (app) => {
  app.component(Dropdown.name, Dropdown);
};

export { Dropdown as default };
