import { defineComponent, computed, ref, createVNode, createTextVNode, mergeProps } from 'vue';
import { getComponent } from '../_util/props-util/index.js';
import inputProps from './inputProps.js';
import Button from '../button/index.js';
import Input from './input.js';

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
const prefixCls = "x-input-search";
var InputSearch = defineComponent({
  name: prefixCls,
  components: {
    Button,
    Input
  },
  props: __spreadProps(__spreadValues({}, inputProps), {
    enterButton: String || Boolean
  }),
  setup(props, {
    attrs,
    emit,
    expose,
    slots
  }) {
    const showBtn = computed(() => !!props.enterButton || props.enterButton === "" || !!slots.enterButton);
    const btnText = computed(() => {
      let text = "\u{1F50D}";
      if (typeof props.enterButton === "string") {
        text = props.enterButton || text;
      }
      return text;
    });
    const btnIcon = computed(() => {
      if (props.enterButton === "")
        return "search";
      if (props.enterButton === true)
        return "search";
    });
    const inputRef = ref();
    function onSearch() {
      emit("search", inputRef.value.value);
      inputRef.value.focus();
    }
    expose({
      focus() {
        inputRef.value.focus();
      },
      blur() {
        inputRef.value.blur();
      },
      get value() {
        return inputRef.value.value;
      }
    });
    return (vm) => {
      const cls = [prefixCls, {
        [`${prefixCls}-enter`]: showBtn.value
      }];
      let enterNode = showBtn.value && (getComponent(vm, "enterButton") || createVNode(Button, {
        "type": "primary"
      }, {
        default: () => [btnText.value]
      }));
      if (!enterNode)
        enterNode = createTextVNode("\u{1F50D}");
      const _enterNode = function() {
        return enterNode;
      }();
      enterNode = createVNode("div", {
        "class": `${prefixCls}_enter`,
        "onClick": onSearch
      }, [enterNode]);
      return createVNode(Input, mergeProps({
        "class": cls,
        "ref": inputRef
      }, __spreadValues(__spreadValues({}, props), attrs), {
        "onPressEnter": onSearch
      }), {
        addonAfter: () => showBtn.value && enterNode,
        suffix: () => !showBtn.value && enterNode
      });
    };
  }
});

export { InputSearch as default };
