import { defineComponent, computed, createVNode, mergeProps } from 'vue';
import '../tooltip/index.js';
import MenuItemContent from './MenuItemContent.js';
import { useMenu } from './utils.js';
import Tooltip from '../tooltip/Tooltip.js';

const menuItemProps = {
  label: [String, Object],
  value: [String, Number],
  icon: [Object, Function],
  disabled: Boolean
};
var MenuItem = defineComponent({
  name: "x-menu-item",
  props: menuItemProps,
  setup(props) {
    const {
      paddingLeftRef,
      menuContext,
      disabledRef,
      isActiveRef,
      isRoot
    } = useMenu(props);
    const prefixCls = "x-menu-item";
    const selectedRef = computed(() => menuContext.valueRef.value === props.value);
    function onClick() {
      menuContext.onSelect(props);
    }
    return () => {
      const cls = {
        [prefixCls]: true,
        [`${prefixCls}-selected`]: selectedRef.value,
        [`${prefixCls}-disabled`]: disabledRef.value
      };
      return createVNode(Tooltip, {
        "disabled": !isRoot || disabledRef.value || !menuContext.collapsedRef.value,
        "placement": "right",
        "title": props.label
      }, {
        default: () => [createVNode("div", {
          "class": cls,
          "role": "menuitem",
          "aria-disabled": disabledRef.value
        }, [createVNode(MenuItemContent, mergeProps(props, {
          "style": `paddingLeft:${paddingLeftRef.value}px`,
          "active": isActiveRef.value,
          "onClick": onClick
        }), null)])]
      });
    };
  }
});

export { MenuItem as default, menuItemProps };
