import { defineComponent, createVNode, cloneVNode, resolveDirective, withDirectives } from 'vue';
import { render } from '../_util/props-util/index.js';
import { useMenu } from './utils.js';
import BorderLight from '../border-light/index.js';
import '../icon/index.js';
import Icon from '../icon/Icon.js';

var MenuItemContent = defineComponent({
  name: "x-menu-item-content",
  directives: {
    [BorderLight.name]: BorderLight
  },
  props: {
    node: Object,
    label: [String, Object],
    value: [String, Number],
    icon: [Object, Function],
    disabled: Boolean,
    showArrow: Boolean,
    active: Boolean,
    collapsed: Boolean
  },
  setup(props) {
    const {
      indentRef,
      menuContext
    } = useMenu(props);
    return () => {
      const indent = indentRef.value;
      const prefixCls = "x-menu-item-content";
      const cls = {
        [prefixCls]: true,
        [`${prefixCls}-active`]: props.active,
        [`${prefixCls}-disabled`]: props.disabled,
        [`${prefixCls}-collapsed`]: props.collapsed
      };
      const iconNode = render(props.icon);
      let vnode = createVNode("div", {
        "class": cls
      }, [iconNode && cloneVNode(iconNode, {
        class: `${prefixCls}_icon`,
        style: `marginLeft:-${indent}px`,
        role: "none"
      }), createVNode("span", {
        "class": `${prefixCls}_label`
      }, [props.label]), props.showArrow && createVNode(Icon, {
        "class": `${prefixCls}_arrow`,
        "type": "arrow-down"
      }, null)]);
      if (menuContext.props.borderLight) {
        const dir = resolveDirective("border-light");
        const dirVal = {
          borderWidth: 1.2,
          borderGradientSize: 65,
          style: {
            opacity: 0.8
          }
        };
        vnode = withDirectives(vnode, [[dir, dirVal]]);
      }
      return vnode;
    };
  }
});

export { MenuItemContent as default };
