import { defineComponent, provide, createVNode } from 'vue';
import { useMenu, createItem } from './utils.js';
import { SubmenuContextKey } from './Submenu.js';

const submenuProps = {
  node: Object,
  label: [String, Object],
  value: [String, Number],
  children: Array,
  icon: [Object, Function],
  disabled: Boolean
};
var MenuItemGroup = defineComponent({
  name: "x-menu-item-group",
  props: submenuProps,
  setup(props) {
    const {
      paddingLeftRef,
      parentPaddingLeftRef
    } = useMenu(props);
    provide(SubmenuContextKey, {
      props,
      paddingLeftRef: parentPaddingLeftRef
    });
    return () => {
      const prefixCls = "x-menu-item-group";
      const childrenNode = createVNode("div", {
        "class": `${prefixCls}_children`
      }, [props.children.map(createItem)]);
      return createVNode("div", {
        "class": prefixCls
      }, [createVNode("div", {
        "class": `${prefixCls}_label`,
        "style": `padding-left:${paddingLeftRef.value}px`
      }, [props.label]), childrenNode]);
    };
  }
});

export { MenuItemGroup as default, submenuProps };
