import { defineComponent, ref, watch, toRef, onMounted, watchPostEffect, onUnmounted, createVNode, createTextVNode, cloneVNode, Teleport, withDirectives, Transition, vShow } from 'vue';
import { getSlot, getComponent } from '../_util/props-util/index.js';
import Card from '../card/index.js';
import Button from '../button/index.js';
import Space from '../space/index.js';

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
const modalProps = {
  visible: Boolean,
  icon: Object,
  title: String,
  width: {
    default: 512
  },
  bodyStyle: [String, Object],
  bodyClass: String,
  to: {
    type: [String, Object],
    default: "body"
  },
  footer: Boolean,
  ok: {
    default: true
  },
  cancel: {
    default: true
  },
  mask: {
    default: true
  },
  maskClosable: {
    default: true
  },
  closable: {
    default: true
  },
  esc: {
    default: true
  },
  beforeOk: {
    type: Function
  }
};
const prefixCls = "x-modal";
var Modal = defineComponent({
  name: prefixCls,
  inheritAttrs: false,
  props: modalProps,
  emits: ["update:visible", "change", "ok", "cancel", "afterLeave", "maskClick"],
  setup(props, {
    emit,
    slots,
    attrs
  }) {
    const loadingRef = ref(false);
    const displayedRef = ref(props.visible);
    watch(toRef(props, "visible"), (val) => {
      if (val)
        displayedRef.value = val;
    });
    function onAfterLeave() {
      displayedRef.value = false;
      emit("afterLeave");
    }
    const containerRef = ref();
    onMounted(() => {
      containerRef.value.addEventListener("scroll", (e) => {
        e.preventDefault();
        e.stopPropagation();
      });
    });
    function onKeydown(e) {
      if (e.key === "Escape" && props.esc)
        close();
    }
    onMounted(() => {
      watchPostEffect(() => {
        if (props.esc && props.visible)
          document.addEventListener("keydown", onKeydown);
        else
          document.removeEventListener("keydown", onKeydown);
      });
    });
    onUnmounted(() => {
      document.removeEventListener("keydown", onKeydown);
    });
    function close() {
      emit("update:visible", false);
      emit("change", false);
    }
    function onMaskClick(e) {
      console.log("onMaskClick");
      emit("maskClick", e);
      if (!props.maskClosable)
        return;
      if (e.target === e.currentTarget)
        close();
    }
    function onOk() {
      if (props.beforeOk) {
        const ret = props.beforeOk();
        if (ret == null ? void 0 : ret.then) {
          loadingRef.value = true;
          ret.then(() => {
            loadingRef.value = false;
            close();
          });
        } else {
          if (ret !== false)
            close();
        }
      } else {
        close();
      }
      emit("ok");
    }
    function onCancel() {
      close();
      emit("cancel");
    }
    function getFooterNode() {
      if (slots.footer) {
        return slots.footer();
      } else if (props.footer) {
        const loading = loadingRef.value;
        const cancelProps = {
          class: `${prefixCls}_cancel`,
          onClick: onCancel
        };
        const okProps = {
          class: `${prefixCls}_ok`,
          loading,
          autofocus: true,
          onClick: onOk
        };
        let cancelBtn = slots.cancel ? getSlot(slots.cancel)[0] : props.cancel && createVNode(Button, null, {
          default: () => [createTextVNode("\u53D6\u6D88")]
        });
        let okBtn = slots.ok ? getSlot(slots.ok)[0] : props.ok && createVNode(Button, {
          "type": "primary"
        }, {
          default: () => [createTextVNode("\u786E\u8BA4")]
        });
        cancelBtn = cancelBtn ? cloneVNode(cancelBtn, cancelProps) : void 0;
        okBtn = okBtn ? cloneVNode(okBtn, okProps) : void 0;
        return createVNode(Space, {
          "justify": "end"
        }, {
          default: () => [cancelBtn, okBtn]
        });
      }
    }
    return (vm) => {
      const {
        to,
        visible,
        width,
        bodyStyle,
        bodyClass,
        closable
      } = props;
      const cardProps = __spreadProps(__spreadValues({}, attrs), {
        class: prefixCls,
        style: {
          width: width + "px"
        },
        closable,
        contentStyle: bodyStyle,
        contentClass: [`${prefixCls}_body`, bodyClass],
        footerClass: `${prefixCls}_footer`,
        role: "document",
        onClose: onCancel
      });
      return createVNode(Teleport, {
        "to": to
      }, {
        default: () => [withDirectives(createVNode("div", {
          "class": `${prefixCls}-container`,
          "ref": containerRef
        }, [createVNode(Transition, {
          "name": "fade",
          "appear": true
        }, {
          default: () => [withDirectives(createVNode("div", {
            "class": [`${prefixCls}-mask`, props.mask && "show"],
            "onClick": onMaskClick
          }, null), [[vShow, props.visible]])]
        }), createVNode(Transition, {
          "name": prefixCls,
          "appear": true,
          "onAfterLeave": onAfterLeave
        }, {
          default: () => [withDirectives(createVNode(Card, cardProps, {
            icon: () => getComponent(vm, "icon"),
            title: () => getComponent(vm, "title"),
            default: slots.default,
            footer: getFooterNode
          }), [[vShow, visible]])]
        })]), [[vShow, displayedRef.value]])]
      });
    };
  }
});

export { Modal as default, prefixCls };
