import { defineComponent, getCurrentInstance, createVNode, computed } from 'vue';
import { getComponent } from '../_util/props-util/index.js';
import '../icon/index.js';
import Icon from '../icon/Icon.js';

const preCls = "x-progress";
const Progress = defineComponent({
  name: preCls,
  props: {
    percent: {
      type: Number,
      default: 0
    },
    color: [String, Object, Function],
    strokeWidth: {
      type: Number,
      default: 8
    },
    railColor: [String, Object, Function],
    railStyle: [String, Object, Array],
    status: {
      type: String,
      default: "default"
    },
    textInside: Boolean,
    showText: {
      type: Boolean,
      default: true
    },
    format: Function
  },
  setup(props) {
    const {
      proxy: vm
    } = getCurrentInstance();
    const iconMap = {
      success: "check-circle",
      error: "close-circle",
      warning: "info-circle"
    };
    function getStatus() {
      return props.percent >= 100 ? "success" : props.status;
    }
    function getText() {
      if (!props.showText)
        return;
      const status = getStatus();
      const clazz = `${preCls}_text`;
      const text = getComponent(vm, "format", props.percent), mergeText = text || props.percent + "%";
      if (props.textInside || text || status === "default") {
        return createVNode("span", {
          "class": clazz
        }, [mergeText]);
      } else {
        return createVNode(Icon, {
          "class": clazz,
          "type": iconMap[status]
        }, null);
      }
    }
    function getColor(color) {
      var _a;
      if (!color)
        return;
      switch (typeof color) {
        case "string":
          return color;
        case "object":
          return (_a = Object.entries(color).sort((a, b) => +b[0] - +a[0]).find((e) => props.percent >= +e[0])) == null ? void 0 : _a[1];
        case "function":
          return color(props.percent);
      }
    }
    const colorRef = computed(() => getColor(props.color));
    const railColorRef = computed(() => getColor(props.railColor));
    return () => {
      const {
        percent,
        textInside,
        strokeWidth,
        railStyle
      } = props;
      const _percent = Math.min(100, Math.max(0, percent));
      const color = colorRef.value, railColor = railColorRef.value;
      const clazz = [preCls, `${preCls}-status-${getStatus()}`, {
        [`${preCls}-text-inside`]: textInside
      }];
      return createVNode("div", {
        "class": clazz
      }, [createVNode("div", {
        "class": `${preCls}_rail`,
        "style": [railStyle, {
          background: railColor
        }]
      }, [createVNode("div", {
        "class": `${preCls}_fill`,
        "style": {
          width: `${_percent}%`,
          minHeight: `${strokeWidth}px`,
          background: color
        }
      }, [textInside && getText()])]), textInside || getText()]);
    };
  }
});
Progress.install = (app) => {
  app.component(Progress.name, Progress);
};

export { Progress as default };
