import { onMounted, onBeforeUnmount } from 'vue';
import { findDOMNode } from '../../_util/props-util/index.js';

function useSelectTriggerControl(refs, triggerOpen) {
  function onGlobalMouseDown(event) {
    let target = event.target;
    if (target.shadowRoot && event.composed) {
      target = event.composedPath()[0] || target;
    }
    const elements = refs.map((e) => findDOMNode(e.value));
    const isOutside = elements.every((element) => !(element == null ? void 0 : element.contains(target)));
    if (isOutside) {
      triggerOpen(false);
    }
  }
  onMounted(() => {
    window.addEventListener("mousedown", onGlobalMouseDown);
  });
  onBeforeUnmount(() => {
    window.removeEventListener("mousedown", onGlobalMouseDown);
  });
}

export { useSelectTriggerControl as default };
