import { shallowReactive, watchPostEffect, normalizeStyle, h, render, Fragment } from 'vue';
import { itemRefPrecls, itemPreCls } from './interface.js';
import resizeobserver from '../_util/Dom/resizeobserver.js';
import { getLoading, patchClass, getClass, allClass } from './utils.js';
import { stringifyStyle } from '@vue/shared';

const setup = (el, binding, vnode) => {
  const props = shallowReactive({
    el,
    binding,
    fragment: el.appendChild(document.createElement("div")),
    targets: shallowReactive([]),
    vnode
  });
  el = null;
  binding = null;
  vnode = null;
  const unwatch1 = watchPostEffect(() => {
    var _a;
    const { el: el2, binding: binding2, vnode: vnode2 } = props;
    if (getLoading(binding2)) {
      patchClass(el2, getClass(binding2), allClass(binding2));
      el2.style.cssText = stringifyStyle(normalizeStyle([el2.style.cssText, binding2.value.loadingStyle]));
    } else {
      el2.classList.remove(...allClass(binding2).split(" "));
      el2.style.cssText = stringifyStyle((_a = vnode2.props) == null ? void 0 : _a.style);
    }
  });
  const unwatch2 = watchPostEffect(() => {
    const { binding: binding2, fragment } = props;
    const targets = props.targets, loading = getLoading(binding2);
    if (loading)
      resizeobserver.observer(props.el);
    else
      resizeobserver.unobserve(props.el);
    if (loading) {
      targets.forEach((e) => e.el.classList.add(itemRefPrecls));
    } else {
      targets.forEach((e) => e.el.classList.remove(itemRefPrecls));
    }
    if (loading) {
      const items = targets.map((e) => {
        var _a, _b;
        return h("div", {
          class: [props.binding.value.class, (_a = e.binding.value) == null ? void 0 : _a.class, itemPreCls],
          style: [props.binding.value.style, (_b = e.binding.value) == null ? void 0 : _b.style, e.state.style]
        });
      });
      render(h(Fragment, items), fragment);
    } else {
      render(null, fragment);
    }
  });
  props.unwatch = () => {
    unwatch1();
    unwatch2();
  };
  return props;
};
const skeletonList = [];
const Skeleton = {
  created(el, binding, vnode) {
    skeletonList.push(setup(el, binding, vnode));
  },
  beforeMount(el, binding, vnode) {
    const props = skeletonList.find((e) => e.el === el);
    props.binding = binding;
    props.vnode = vnode;
  },
  beforeUpdate(el, binding, vnode) {
    const props = skeletonList.find((e) => e.el === el);
    props.vnode = vnode;
    props.binding = binding;
  },
  beforeUnmount(el) {
    const i = skeletonList.findIndex((e) => e.el === el);
    if (i < 0)
      return;
    const props = skeletonList.splice(i, 1)[0];
    props.unwatch();
    props.fragment.parentNode.removeChild(props.fragment);
    resizeobserver.dispose(el);
  }
};

export { Skeleton as default, skeletonList };
