import { defineComponent, getCurrentInstance, computed, createVNode, mergeProps } from 'vue';
import useMemo from '../_util/hooks/useMemo.js';
import { getComponent } from '../_util/props-util/index.js';

const prefixCls = "x-switch";
const Switch = defineComponent({
  name: "x-switch",
  props: {
    value: [Boolean, Number, String],
    disabled: Boolean,
    checked: [String, Object],
    unchecked: [String, Object],
    checkedValue: {
      type: [Boolean, Number, String],
      default: true
    },
    uncheckedValue: {
      type: [Boolean, Number, String],
      default: false
    }
  },
  emits: ["update:value", "change"],
  setup(props, {
    emit
  }) {
    const {
      proxy: vm
    } = getCurrentInstance();
    const valueRef = useMemo(() => props.value);
    const checkedRef = computed(() => valueRef.value === props.checkedValue);
    function setChecked(checked) {
      if (props.disabled)
        return;
      valueRef.value = checked ? props.checkedValue : props.uncheckedValue;
      emit("update:value", valueRef.value);
      emit("change", valueRef.value);
    }
    function onClick() {
      setChecked(!checkedRef.value);
    }
    return () => {
      const cls = [prefixCls, {
        [`${prefixCls}-checked`]: checkedRef.value,
        [`${prefixCls}-disabled`]: props.disabled
      }];
      const switchProps = {
        class: cls,
        disabled: props.disabled,
        role: "switch",
        "aria-checked": checkedRef.value
      };
      const node = getComponent(vm, checkedRef.value ? "checked" : "unchecked");
      const prefixNode = node ? createVNode("div", {
        "class": `${prefixCls}-text`
      }, [node]) : void 0;
      return createVNode("button", mergeProps(switchProps, {
        "type": "button",
        "onClick": onClick
      }), [createVNode("div", {
        "class": `${prefixCls}_core`
      }, null), prefixNode]);
    };
  }
});
Switch.install = (app) => {
  app.component(Switch.name, Switch);
};

export { Switch as default };
