import { isVNode, defineComponent, inject, createVNode, mergeProps } from 'vue';
import { dataTableContextKey } from './interface.js';
import { getRowClassName, getColumnClassName } from './utils.js';
import VirtualList from '../virtual-list/index.js';
import TableHeader from './TableHeader.js';

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !isVNode(s);
}
const TableBody = defineComponent({
  props: {
    showHeader: Boolean
  },
  setup(props, {
    slots
  }) {
    const {
      props: tableProps,
      leftFixedColumsRef,
      rightFixedColumsRef,
      fixedColumnLeftMapRef,
      fixedColumnRightMapRef
    } = inject(dataTableContextKey);
    const {
      prefixCls
    } = tableProps;
    const itemsWrap = (_, {
      slots: slots2
    }) => createVNode("div", null, [createVNode("table", {
      "class": `${prefixCls}_table`,
      "style": `width:${tableProps.scrollX}px`
    }, [props.showHeader && createVNode(TableHeader, null, null), createVNode("tbody", {
      "class": `${prefixCls}_tbody`
    }, __spreadValues({}, slots2))])]);
    return () => {
      const {
        prefixCls: prefixCls2,
        columns,
        data,
        rowProps,
        rowClass,
        maxHeight,
        virtualScroll
      } = tableProps;
      const scrollableY = !!maxHeight;
      const rows = data.map((row, index) => {
        const cls = [`${prefixCls2}_tr`, getRowClassName(rowClass, row, index)];
        const props2 = rowProps == null ? void 0 : rowProps(row, index);
        return createVNode("tr", mergeProps({
          "class": cls,
          "key": row.key
        }, props2), [columns.map((col, colIndex) => {
          const fixLast = col === leftFixedColumsRef.value[leftFixedColumsRef.value.length - 1], rightFirst = col === rightFixedColumsRef.value[0];
          const cls2 = [`${prefixCls2}_td`, getColumnClassName(col, colIndex), {
            [`${prefixCls2}_td-fix-${col.fixed}`]: col.fixed,
            [`${prefixCls2}_td-fix-left-last`]: fixLast,
            [`${prefixCls2}_td-fix-right-first`]: rightFirst,
            [`${prefixCls2}_td-align-${col.align}`]: col.align
          }];
          const style2 = {
            width: col.width + "px",
            left: fixedColumnLeftMapRef.value[col.key] + "px",
            right: fixedColumnRightMapRef.value[col.key] + "px"
          };
          return createVNode("td", {
            "key": col.key,
            "class": cls2,
            "style": style2
          }, [slots[col.key] ? slots[col.key]({
            row,
            index
          }) : row[col.key]]);
        })]);
      });
      let tableNode = createVNode(itemsWrap, null, _isSlot(rows) ? rows : {
        default: () => [rows]
      });
      const style = [scrollableY && `max-height:${maxHeight}px;`];
      if (!virtualScroll)
        style.push({
          overflowY: "scroll"
        });
      const bodyProps = {
        class: `${prefixCls2}_body`,
        style
      };
      if (virtualScroll) {
        tableNode = createVNode(VirtualList, mergeProps({
          "items": rows,
          "itemsWrap": itemsWrap,
          "virtual": true,
          "itemKey": "key"
        }, bodyProps), {
          default: ({
            item
          }) => item
        });
      }
      return createVNode("div", bodyProps, [tableNode]);
    };
  }
});

export { TableBody as default };
