import { inject, createVNode } from 'vue';
import { dataTableContextKey } from './interface.js';
import { getColumnClassName, renderTitle } from './utils.js';

const TableHeader = (props) => {
  const {
    props: tableProps,
    leftFixedColumsRef,
    rightFixedColumsRef,
    fixedColumnLeftMapRef,
    fixedColumnRightMapRef
  } = inject(dataTableContextKey);
  const {
    prefixCls,
    columns,
    scrollX,
    virtualScroll
  } = tableProps;
  const theadNode = createVNode("thead", {
    "class": `${prefixCls}_thead`
  }, [createVNode("tr", {
    "class": `${prefixCls}_tr`
  }, [columns.map((col, colIndex) => {
    const fixLast = col === leftFixedColumsRef.value[leftFixedColumsRef.value.length - 1], rightFirst = col === rightFixedColumsRef.value[0];
    const cls = [`${prefixCls}_th`, getColumnClassName(col, colIndex), col.fixed && `${prefixCls}_th-fix-${col.fixed}`, fixLast && `${prefixCls}_td-fix-left-last`, rightFirst && `${prefixCls}_td-fix-right-first`, col.align && `${prefixCls}_th-align-${col.align}`];
    const style2 = {
      width: col.width + "px",
      left: fixedColumnLeftMapRef.value[col.key] + "px",
      right: fixedColumnRightMapRef.value[col.key] + "px"
    };
    return createVNode("th", {
      "class": cls,
      "style": style2
    }, [renderTitle(col)]);
  })])]);
  if (!props.discrete) {
    return theadNode;
  }
  const style = [];
  if (virtualScroll)
    style.push({
      overflowY: "hidden"
    });
  return createVNode("div", {
    "class": `${prefixCls}_header`,
    "style": style
  }, [createVNode("table", {
    "class": `${prefixCls}_table`,
    "style": `width:${scrollX}px`
  }, [theadNode])]);
};

export { TableHeader as default };
