import { defineComponent, getCurrentInstance, inject, onBeforeUnmount, computed, createVNode } from 'vue';
import { remove } from '@vue/shared';
import { computedEager } from '@vueuse/core';
import asd from '../_internal/display-directive/index.js';
import { TabsContextKey } from './interface.js';
import { getKey } from './utils.js';

const preCls = "x-tab-pane";
const tabPaneProps = {
  label: [String, Number],
  name: [String, Number],
  displayDirective: String
};
var TabPane = defineComponent({
  name: preCls,
  props: tabPaneProps,
  setup(props, {
    slots
  }) {
    const proxy = getCurrentInstance().proxy;
    const tabsContext = inject(TabsContextKey);
    tabsContext.tabpanes.value.push(proxy);
    onBeforeUnmount(() => remove(tabsContext.tabpanes.value, proxy));
    const key = computed(() => {
      var _a;
      return (_a = getKey(proxy)) != null ? _a : tabsContext.tabpanes.value.indexOf(proxy);
    });
    const visible = computedEager(() => key.value == tabsContext.value.value);
    return () => createVNode(asd, {
      "type": props.displayDirective,
      "visible": visible.value && !!slots.default
    }, {
      default: () => {
        var _a;
        return [createVNode("div", {
          "class": preCls
        }, [(_a = slots.default) == null ? void 0 : _a.call(slots)])];
      }
    });
  }
});

export { TabPane as default };
