import { defineComponent, toRef, ref, onMounted, provide, createVNode } from 'vue';
import '../_util/hooks/index.js';
import TabPane from './TabPane.js';
import Tabbar from './Tabbar.js';
import { getKey, normalize } from './utils.js';
import { TabsContextKey } from './interface.js';
import useNamespace from '../_util/hooks/useNameSpace2.js';
import useMergedState from '../_util/hooks/useMergedState.js';

var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
const preCls = "x-tabs";
var Tabs = defineComponent({
  name: preCls,
  props: {
    value: [String, Number],
    defaultValue: [String, Number],
    size: {
      type: String,
      default: "medium"
    },
    barStyle: [String, Object, Array],
    justify: String,
    type: {
      type: String,
      default: "bar"
    },
    beforeLeave: Function,
    tabs: Array
  },
  emits: ["update:value"],
  setup(props, {
    emit,
    slots
  }) {
    var _a;
    const ns = useNamespace(preCls);
    const valueRef = useMergedState(toRef(props, "value"), ref((_a = props.value) != null ? _a : props.defaultValue));
    onMounted(() => {
      var _a2;
      if (valueRef.value == null)
        valueRef.value = (_a2 = getKey(tabpanes.value[0])) != null ? _a2 : 0;
    });
    const tabpanes = ref([]);
    provide(TabsContextKey, {
      value: valueRef,
      tabpanes
    });
    function onChange(key) {
      return __async(this, null, function* () {
        if (valueRef.value === key)
          return;
        if (typeof props.beforeLeave === "function") {
          const bool = yield props.beforeLeave(key, valueRef.value);
          if (!bool)
            return;
        }
        valueRef.value = key;
        emit("update:value", key);
      });
    }
    return () => {
      var _a2, _b;
      return createVNode("div", {
        "class": [ns.b(), ns.m(props.type), ns.m(props.size)]
      }, [createVNode(Tabbar, {
        "value": valueRef,
        "tabs": tabpanes,
        "type": props.type,
        "onChange": onChange,
        "style": `justify-content: ${props.justify}`
      }, null), (_a2 = slots.default) == null ? void 0 : _a2.call(slots), (_b = props.tabs) == null ? void 0 : _b.map((e) => createVNode(TabPane, normalize(e), null))]);
    };
  }
});

export { Tabs as default };
