import { defineComponent, getCurrentInstance, reactive, watch, createVNode } from 'vue';
import { normalizeClass } from '@vue/shared';
import { getComponent } from '../_util/props-util/index.js';
import '../trigger/index.js';
import { placements } from './placements.js';
import tooltipProps from './tooltipProps.js';
import Trigger from '../trigger/Trigger.js';

var Tooltip = defineComponent({
  name: "x-tooltip",
  props: tooltipProps,
  slots: ["title"],
  emits: ["update:visible", "visibleChange"],
  setup(props, {
    emit,
    slots
  }) {
    const {
      proxy: vm
    } = getCurrentInstance();
    const {
      prefixCls,
      builtinPlacements
    } = props;
    const state = reactive({
      visible: props.visible
    });
    watch(() => props.visible, (v) => {
      state.visible = v;
    });
    function onPopupVisibleChange(v) {
      state.visible = v;
      emit("update:visible", v);
      emit("visibleChange", v);
    }
    function getPopupElement() {
      return getComponent(vm, "title");
    }
    return () => {
      const popupClass = normalizeClass([`${prefixCls} ${prefixCls}-placement-${props.placement}`, props.popupClass]);
      const triggerProps = {
        action: props.trigger,
        disabled: props.disabled,
        popup: getPopupElement(),
        popupClass,
        popupTransitionName: props.transitionName || prefixCls,
        popupAlign: props.align || (builtinPlacements == null ? void 0 : builtinPlacements[props.placement]) || placements[props.placement],
        getPopupContainer: props.getPopupContainer,
        onPopupVisibleChange,
        mouseEnterDelay: props.mouseEnterDelay,
        mouseLeaveDelay: props.mouseLeaveDelay,
        popupStyle: props.popupStyle
      };
      return createVNode(Trigger, triggerProps, {
        default: () => {
          var _a;
          return [(_a = slots.default) == null ? void 0 : _a.call(slots)];
        }
      });
    };
  }
});

export { Tooltip as default };
