import { defineComponent, ref, onMounted, toRef, computed, nextTick, createVNode } from 'vue';
import '../_util/css/index.js';
import { getEL, getComponent, _cloneVNode } from '../_util/props-util/index.js';
import '../_util/hooks/index.js';
import Resizable from '../drag-resize/index.js';
import '../icon/index.js';
import useMergedState from '../_util/hooks/useMergedState.js';
import formatLength from '../_util/css/format-length.js';
import Icon from '../icon/Icon.js';

const windowProps = {
  title: [String, Object],
  prefix: [String, Object],
  suffix: [String, Object],
  draggable: {
    type: Boolean,
    default: true
  },
  closable: {
    type: Boolean,
    default: true
  },
  fullscreen: {
    type: Boolean,
    default: void 0
  },
  resizable: {
    type: Boolean,
    default: true
  },
  w: Number,
  h: Number,
  type: {
    type: String,
    default: "default"
  },
  headerStyle: [String, Array, Object],
  bodyStyle: [String, Array, Object],
  footer: [Function, Object, Array]
};
const prefixCls = "x-window";
var Window = defineComponent({
  name: prefixCls,
  props: windowProps,
  // prettier-ignore
  emits: ["close", "resize", "dragging", "resizing", "update:width", "update:height", "update:x", "update:y", "update:w", "update:h", "update:fullscreen", "resizing", "resize-start", "resize-end", "activated", "deactivated"],
  setup(props, {
    emit,
    slots
  }) {
    const elRef = ref();
    const headerRef = ref();
    onMounted(() => {
      getEL(elRef.value).addEventListener("mousedown", (e) => getEL(elRef.value).focus());
    });
    const mergeFullscreen = useMergedState(toRef(props, "fullscreen"), ref(false), (val) => emit("update:fullscreen", val));
    const styleRef = computed(() => {
      return mergeFullscreen.value ? `` : `width: ${formatLength(props.h)}; height: ${formatLength(props.w)}`;
    });
    let lastStyle = "";
    let style = "";
    function onToggleSize() {
      if (!props.resizable)
        return;
      if (!mergeFullscreen.value) {
        lastStyle = getEL(elRef.value).getAttribute("style");
      } else {
        style = lastStyle;
        nextTick(() => style = "");
      }
      mergeFullscreen.value = !mergeFullscreen.value;
    }
    function onClose() {
      emit("close");
    }
    function prevent(e) {
      e.stopPropagation();
    }
    return (vm) => {
      return createVNode(Resizable, {
        "class": [prefixCls, `${prefixCls}-${props.type}`, {
          [`${prefixCls}-resizable`]: props.resizable,
          [`${prefixCls}-fullscreen`]: mergeFullscreen.value
        }],
        "style": style,
        "ref": elRef,
        "dragTarget": headerRef,
        "type": "simple",
        "tabindex": "0"
      }, {
        default: () => {
          var _a;
          return [createVNode("div", {
            "class": `${prefixCls}_header`,
            "style": props.headerStyle,
            "ref": headerRef
          }, [getComponent(vm, "prefix"), createVNode("div", {
            "class": `${prefixCls}_title`
          }, [getComponent(vm, "title")]), createVNode("div", {
            "class": `${prefixCls}_suffix`
          }, [getComponent(vm, "suffix")]), props.resizable && createVNode(Icon, {
            "class": `${prefixCls}_operation`,
            "type": "Rectangle",
            "onMousedown": prevent,
            "onClick": onToggleSize
          }, null), props.closable && createVNode(Icon, {
            "class": `${prefixCls}_operation ${prefixCls}_close`,
            "type": "cross",
            "onMousedown": prevent,
            "onClick": onClose
          }, null)]), createVNode("div", {
            "class": `${prefixCls}_body`,
            "style": props.bodyStyle
          }, [(_a = slots.default) == null ? void 0 : _a.call(slots)]), _cloneVNode(getComponent(vm, "footer"), {
            class: `${prefixCls}_footer`
          })];
        }
      });
    };
  }
});

export { Window as default };
